dtxFramework.DTXElement = function(framework, id) {

	this.$dtxVersion = "1.0";

	this.$id = id;
	this.$templateId = null;
	this.$document = null;
	this.$framework = framework;
	this.$domNode = null;
	this.$attributes = new Array();
	this.$events = new Array();
	this.$updateDisabled = 0;
	this.$resizeable = false;
	this.$isPopup = false;
	this.$resizeTarget = null;
	this.$domContent = null;
	this.$designMode = false;
	this.$designable = true;
	this.$updateWhenChanged = true;
	this.$isTopElement = false;
	this.$updateId = 0;
	this.$prevDisplay = null;
	this.$notifiedCreation = false;
	this.$orientation = framework.$ORIENT_HORZ;
	this.$templateApplied = false;	
	this.$forceTemplate = false;
	this.$autohidden = "autohidden";
	this.$localeAttributes = [];
	this.$elementIndexes = new Array ();
	this.$recurseResult = null;
	
	if (dtxFramework.IE){
		this.setAttribute("ie", "true");
		//IE version
		var ieVersion = framework.getBrowserVersion();
		this.$isIE6 = (ieVersion >= 5.5) && (ieVersion < 7);
	}
	if (dtxFramework.MOZILLA)
		this.setAttribute("mozilla", "true");
	
	this.childNodes = new Array();
	this.parentNode = null;
	this.tagName = undefined;
	this.framework = framework;
	this.boxObject = {left: -1, top: -1, width: -1, height: -1};
	
	var self = this;
	
	this.isVisible = function() {
		return (this.$isTopElement || this.parentNode != null && this.parentNode.isVisible());
	};
	
	this.$needsUpdate = function() {
		return (this.$updateDisabled == 0) && (this.$isTopElement || this.parentNode != null && this.parentNode.$needsUpdate());
	};
	
	this.appendChild = function(element) {
		if (!framework.isNativeElement(element)) {
			this.childNodes.push(element);
			element.$setParent(this);
			if (this.$designMode) {
				element.$setDesignMode(true);
			}
			if (this.$updateWhenChanged && this.$needsUpdate())
				this.$update();
		}
		else {
			this.$domContent = element;
			if (this.$domNode != null) {
				while (this.$domNode.childNodes.length > 0)
					this.$domNode.removeChild(this.$domNode.firstChild);
				this.$domNode.appendChild(element);
			}
		}
	};
	
	
	this.insertBefore = function (element, new_node) {
		if (!framework.isNativeElement(element)) {
			var idx = 0;
			var elmidx = 0 ;
			var newidx = 0 ;
			var remidx = 0 ;
			
				
			element.$setParent(this);
			if (this.$designMode) {
				element.$setDesignMode(true);
			}
			while (idx < this.childNodes.length) {
					if (this.childNodes[idx] == element){
						elmidx = idx;
					}
					if (this.childNodes[idx] == new_node){
						newidx = idx;
					}
					idx++;
			}

			this.childNodes.splice(newidx , 1 );
			this.childNodes.splice(elmidx, 0  , new_node);

			for (var i=0; i<this.childNodes.length; i++){
				this.childNodes[i].setAttribute("itemindex", i);			
				this.childNodes[i].setAttribute("index", i);
			}
						
			
			if (element.$domNode != null && element.$domNode.parentNode != null && new_node.$domNode != null) {
				if (elmidx == 0  || elmidx == 1 || newidx - elmidx == 1){
					element.$domNode.parentNode.insertBefore(new_node.$domNode, element.$domNode);
				} else {
					element.$domNode.parentNode.insertBefore(new_node.$domNode, element.$domNode.nextSibling);
				}
			}
			
			
			if (this.$updateWhenChanged)
				this.$update();
		}
	};
	
	
	this.removeChild = function(element) {
		if (!framework.isNativeElement(element)) {
			var idx = 0;
			while (idx < this.childNodes.length && this.childNodes[idx] != element) 
				idx++;
			if (idx < this.childNodes.length) {
				this.childNodes.splice(idx, 1);
				element.$setParent(null);
				try {
					if (element.$domNode != null && element.$domNode.parentNode != null) {
						element.$domNode.parentNode.removeChild(element.$domNode);
					}
				}
				catch (e) {};
				if (this.$updateWhenChanged)
					this.$update();
			}
		}
		else
			if (element.$domNode != null)
				this.$domNode.removeChild(element);
	};
	
	this.removeAll = function() {
		this.$domContent = null;
		if (dtxFramework.MOZILLA) {
			//$MOZILLA_START$//
			this.childNodes = new Array();
			if (this.$domNode != null) {
				while (this.$domNode.childNodes.length > 0) {
					this.$domNode.removeChild(this.$domNode.childNodes[0]);
				}
			}
			//$MOZILLA_END$//
		}
	};
	
	this.$init = function() {
	};
	
	this.$setDOM = function(dom) {
		this.$document = dom;
	};
	
	this.$getNode = function(nodeId) {
		return this.$document.getElementById(nodeId);
	};
	
	this.$refresh = function() {
		// TODO: set timeout in framework
	};
	
	this.$getCSSProperty = function(rule, property) {
		if (dtxFramework.IE && DTFramework.WEB) {
			/*
			if(this.$framework.$cssProperties[property])
				return this.$framework.$cssProperties[property];
			*/
			var cssText = new String(rule);
			cssText = cssText.replace(/ /g, "");
		}else{
			var cssText = new String(rule.style.cssText);
			cssText = cssText.replace(/ /g, "");
	        cssText = ";" + cssText + ";";
		}	
		
		var index = cssText.indexOf(property + ":");
        if (index === -1) { // IE 6 does not like properties that begin with a dash, as in -dtx
            property = property.substring(1);
            index = cssText.indexOf( property + ":");
        }
		var value = "";
		var c;
		if (index != -1) {
			index += property.length + 1;
			do {
				c = cssText.charAt(index);
				if (c != ";")
					value += c;
				index++;
			}
			while (c != ';');
		}
		
		if (value != "")
			this.$framework.$cssProperties[property] = value;
		return value;
	};
	
	this.$getCSSRule = function(name) {
		try {
			if (dtxFramework.IE && DTFramework.WEB) {
				var rule = framework.getCSSRules(name);					
				return rule;
			}else{	
				if (this.$document.styleSheets.length > 0) {
					var rules = this.$document.styleSheets[0].rules;
					name = "." + name;
					var i = rules.length - 1;
					for (; i >= 0 && rules[i].selectorText != name; i--);
					if (i >= 0)
						return rules[i];
					else
						return null;
				}
				else
					return null;
			}
		}
		catch (e) {
			return null; // TODO: fix access denied error in IE6!
		}
	};
	
	this.$getScreenBox = function() {
		if (this.$domNode != null) {
			if (dtxFramework.IE) {
				//$IE_START$//
				var win;
				if (this.$document != null) {
					win = this.$document.parentWindow;
				}
				else
					win = window;
				var x, y;
				var offsX = win.screenLeft;
				var offsY = win.screenTop;
				var offsParent = this.$domNode;
				while (offsParent != null) {
					offsX += offsParent.offsetLeft;
					offsY += offsParent.offsetTop;
					offsParent = offsParent.offsetParent;
				}
				return {
					left: offsX, 
					top: offsY, 
					width: this.$domNode.offsetWidth, 
					height: this.$domNode.offsetHeight
				};
				//$IE_END$//
			}
			else {
				//$MOZILLA_START$//
				return {left: 0, top: 0, width: 0, height: 0};
				//$MOZILLA_END$//
			}
		}
		else
			return {left: 0, top: 0, width: 0, height: 0};
	};

	this.$loadFromJSON = function(json) {
	
		// presist attribute is special,
		//	it affects how other attributes are handled
		var persist = json.attributes["persist"];
		if (typeof persist == "string" && persist != "") {
			this.setAttribute("persist", persist);
		}

		var localize = json.attributes["localize"];
		if (typeof localize == "string" && localize != "") {
			this.setAttribute("localize", localize);
		}
		
		var localeAttributes = " "+this.localeAttributesToStr()+" ";
		var attrIndex = "";
		
		for (attrId in json.attributes) {
			var attrValue = json.attributes[attrId];
			if (attrId != "persist"){
				if (this.$elementIndexes[attrId] && this.$elementIndexes[attrId] != ""){
					attrIndex = this.$elementIndexes[attrId];
				}
				if(localeAttributes.indexOf(attrId)>-1){
					attrValue = this.framework.localize(json.attributes[attrId],this.$id,attrId,attrIndex);
				}
				this.setAttribute(attrId, attrValue);
			}
		}

		//If it's the toolbar, try to get the locale from persistence
		if(this.tagName == 'toolbar'){
			this.framework.$applyPersistenceToElement(this,'locale');
		}
		for (childId in json.children) {
			var child = json.children[childId];
			
			//FF Review, we need to ensure we are not loading scripts anywhere but locally.
			if (child.type.toLowerCase() == "script" && 
				child.getAttribute("src") !=  null && 
				(child.getAttribute("src").indexOf("http") == 0 || 
				child.getAttribute("src").indexOf("https") == 0 ||
				child.getAttribute("src").indexOf("://")== 0))
				continue;
			else
				child = framework.createElement(json.children[childId].type, childId); 
			child.setAttribute("id", childId);
			this.childNodes.push(child);
			child.$loadFromJSON(json.children[childId]);
			child.$setParent(this);
		}
	};
	
	this.loadFromXML = function(xmlNode) {
		this.$loadFromXML(xmlNode);
	};
	
	this.$loadFromXML = function(xmlNode) {
		var attrLocalize = xmlNode.attributes.getNamedItem('localize');
		var attID = xmlNode.attributes.getNamedItem('id');
		var id = (attID!=null)?attID.value:null;

		if (attrLocalize != null) {
			this.setAttribute("localize", attrLocalize.nodeValue);
		}

		var localeAttributes = " "+this.localeAttributesToStr()+" ";
		var attrIndex = "";
		for (var i = 0; i < xmlNode.attributes.length; i++) {
			var attr = xmlNode.attributes[i];
			var attrValue = attr.value;
			if (this.$elementIndexes[attr.name] && this.$elementIndexes[attr.name] != ""){
				attrIndex = this.$elementIndexes[attr.name];
			}
			this.setAttribute(attr.name, (localeAttributes.indexOf(attr.name)>-1)?this.framework.localize(attrValue,id,attr.name,attrIndex):attrValue);
		}
		for (var i = 0; i < xmlNode.childNodes.length; i++) {
			var node = xmlNode.childNodes[i];
			if (node.nodeType == 1) {
				var childId = node.getAttribute("id");
				if (typeof childId != "string")
					childId = "";
					
				//FF Review, we need to ensure we are not loading scripts anywhere but locally.
                if (node.tagName.toLowerCase() == "script" && 
                    node.getAttribute("src") != null && 
                    (node.getAttribute("src").indexOf("http") == 0 || 
                    node.getAttribute("src").indexOf("https") == 0 ||
                    node.getAttribute("src").indexOf("://")== 0))
			        continue;
			    else
					var child = this.framework.createElement(node.tagName, childId); 
					
				if (child != null) {
					child.setAttribute("id", childId);
					this.childNodes.push(child);
					child.$loadFromXML(node);
					child.$setParent(this);	
				}
			}
		}
	};
	
	this.$saveToXML = function(xmlNode) {
		var nonSerializeArray = new Array();
		var nonSerialize = this.getAttribute("nonserializeattributes");  
		if (typeof nonSerialize == "string" && nonSerialize != "")
			nonSerializeArray = nonSerialize.split(" ");
		
        //non serialize attributes by defaults			
        nonSerializeArray.push("parentWidget", "key_filter", "geoip_filter", "initially_hidden", "initially_optional");			
		
		for (var attrName in this.$attributes) {
			var value = this.$attributes[attrName];
			var idx;
			for (idx = 0; idx < nonSerializeArray.length && nonSerializeArray[idx] != attrName; idx++);
			if (idx == nonSerializeArray.length)
				xmlNode.setAttribute(attrName, value);
		}
		for (var i = 0; i < this.childNodes.length; i++) {
			var child = this.childNodes[i];
			var nonSerialize = child.getAttribute("nonserialize");
			if (nonSerialize != "true") {
				//FF Review, we need to ensure we are not loading scripts anywhere but locally.
                if (child.tagName.toLowerCase() == "script" && 
                    child.getAttribute("src") !=  null && 
                    (child.getAttribute("src").indexOf("http") == 0 || 
                    child.getAttribute("src").indexOf("https") == 0 ||
                    child.getAttribute("src").indexOf("://")== 0))
			        continue;
			    else
					var node = xmlNode.ownerDocument.createElement(child.tagName);
				child.$saveToXML(node);
				xmlNode.appendChild(node);
			}
		}
	};
	
	this.saveToXML = function(xmlNode) {
		this.$saveToXML(xmlNode);
	};
	
	this.reapplyTemplate = function() {
		this.$forceTemplate = true;
		this.$templateApplied = false;
		this.setAttribute("template", this.getAttribute("template"));
	};
	
	this.$applyTemplate = function(templatePath, feedContents) {
			var templateContents = self.framework.loadChromeFile(templatePath);
			if (templateContents == null)
				return;
			
			var feedXML = self.framework.loadXML(feedContents);
	
			if (feedXML != null) {
				var output = self.framework.transformXML(feedContents, templateContents, null);
				var dtxXML = self.framework.loadXML(output);
				self.removeAll();
				self.$attributes = new Array();
				self.$disableUpdates();
				self.$loadFromXML(dtxXML.documentElement);
				self.$enableUpdates();
				self.$update();
				self.$fireEvent("afterapplytemplate",{target:self});
			}
	};
	
	this.loadFromFile = function(filename) {
		var xmlFile = this.framework.loadChromeFile(filename);
		if (xmlFile != null) {
			var xml = this.framework.loadXML(xmlFile);
			if (xml != null)
				this.$loadFromXML(xml.documentElement);
		}
	};
	
	this.getElementById = function(id) {
		this.$recurseResult = null;
		var self = this;
		this.$recurse(this,
			function(element) {
				if (element.$id != id)
					return true;
				else {
					self.$recurseResult = element;
					return false;
				}
			});
		return this.$recurseResult;
	};
	
	this.getElementByTemplateId = function(id) {
		this.$recurseResult = null;
		var self = this;
		this.$recurse(this,
			function(element) {
				if (element.$templateId != id)
					return true;
				else {
					self.$recurseResult = element;
					return false;
				}
			});
		return this.$recurseResult;
	};
	
	this.getContentDocument = function(){
		return this.$document;
	};
	
	this.recurse = function(element, fn) {
		return this.$recurse(element,fn);
	};
	
	this.$recurse = function(element, fn) {
		var result = element;
		if (fn(element)) {
			for (var i = 0; i < element.childNodes.length; i++) {
				var child = element.childNodes[i];
				result = element.$recurse(child, fn);
			}
		}
		return result;
	};
	
	this.$enableUpdates = function() {
		this.$updateDisabled--;
	};
	
	this.$disableUpdates = function() {
		this.$updateDisabled++;
	};
	
	this.$beforeUpdate = function() {
		if (dtxFramework.MOZILLA) {
			//$MOZILLA_START$//
			
			// release DOM nodes
			this.$recurse(self, 
				function(element) {
					if (element != self && element.$deferRender())
						return false;
					if (element.$domNode != null) {
						while (element.$domNode.childNodes.length > 0)
							element.$domNode.removeChild(element.$domNode.childNodes[0]);
						element.$domNode = null;
					}
					if (element.$deferRenderChildren())
						return false;
					else
						return true;
				});
			//$MOZILLA_END$//
		}
		else if (dtxFramework.IE) {
			//$IE_START$//
			this.$recurse(self, 
				function(element) {
					element.$disconnectDOM();
					return true;
				});
			//$IE_END$//	
		}
		
		// assign control indexes
		var nodes = this.childNodes;
		for (var i = 0; i < nodes.length; i++) {
			var element = nodes[i];
			if (element.getAttribute("index") == null)
				element.setAttribute("index", i);
		}
		
		// sort according to index
		this.childNodes.sort(
			function (a, b) {
				var idxA = new Number(a.getAttribute("index"));
				var idxB = new Number(b.getAttribute("index"));
				if (idxA < idxB)
					return -1;
				else if (idxA > idxB)
					return 1;
				else
					return 0;
			}
		);
	
		this.$recurse(self, 
			function(element) {
				if (dtxFramework.IE) {
					//$IE_START$//
					try {
						// save content node
						if (element.$domContent != null && element.$domContent.parentNode != null) {
							element.$domContent.parentNode.removeChild(element.$domContent);
						}
					}
					catch (e) {
					}
					//$IE_END$//
				}
				
				// make sure element has an id
				var id = element.getAttribute("id");
				if (id == null || id == "")
					element.setAttribute("id", element.framework.applicationId + "-" + element.tagName + element.$framework.generateId());
				return true;
			});
	};
	
	this.$assignAnonymousIds = function() {
		this.$recurse(self, 
			function(element) {
				// make sure element has an id
				var id = element.getAttribute("id");
				if (id == null || id == "")
					element.setAttribute("id", element.tagName + element.$framework.generateId());
				return true;
			});
	};
	
	this.$update = function() {
		var updateDisabled = this.$updateDisabled > 0;
		var parent = this.parentNode;
		while (!updateDisabled && parent != null) {
			updateDisabled = parent.$updateDisabled > 0;
			parent = parent.parentNode;
		}
		if (updateDisabled)
			return;
			
		if (this.$document == null)
			this.$setDOM(document);
			
		if (this.$document == null)
			return;
			
		this.$beforeUpdate();
		this.$assignAnonymousIds();

		// render and connect elements
		this.$recurse(self, 
			function(element) {

				if (element != self && element.$deferRender())
					return false;
			
				element.$setDOM(self.$document);
				element.$domNode = element.$document.getElementById(element.$id);
				if (element.$domNode == null) {
					element.$domNode = element.$createDOMNode();
				}
				if (element.$domNode != null) {
					element.$domNode.setAttribute("dtx-obj", element);
					element.$disableUpdates();
					element.$render();
					element.$connectDOM();
					element.$applyAttributes();
					element.$applyStyle();
					if (!element.$designMode)
						element.$hookEvents();
					element.$enableUpdates();
				}
				
				if (element != self && element.$deferRenderChildren())
					return false;
				else
					return true;
			});
		
		this.$recurse(self,
			function(element) {
				if (element != self && element.$deferRender())
					return false;
				var eventObj = {target: element};
				element.$fireEvent("afterupdate", eventObj);
				return true;
			});

	};

	this.$render = function() {
	};
	
	this.$deferRender = function() {
		return false;
	};
	
	this.$deferRenderChildren = function() {
		return false;
	};
	
	this.$getEvent = function(evt) {
		if (!evt) 
			evt = this.$document.parentWindow.event;
		return evt;
	};
	
	this.$getEventTarget = function(evt) {
		if (!evt) 
			evt = this.$document.parentWindow.event;
		var domTarget = null;
		if (dtxFramework.MOZILLA) {
			if (!this.$framework.isNativeElement(evt.target))
				return evt.target;
			else
				domTarget = evt.target;
		}
		else if (dtxFramework.IE)
			if (evt.target)
				return evt.target;
			else
				domTarget = evt.srcElement;
		var target = null;
		while (domTarget != null && target == null) {
			target = domTarget.getAttribute("dtx-obj");
			domTarget = domTarget.parentNode;
		}
		return target;
	};
	
	this.getEvent = function(evt) {
		return this.$getEvent(evt);
	};
	
	this.getEventTarget = function(evt) {
		return this.$getEventTarget(evt);
	};
	
	this.isHidden = function(){
		var hiddenAttr = this.getAttribute("hidden");
		var hidden = typeof hiddenAttr == "string" && hiddenAttr == "true";
		
		var keyFilterAttr = this.getAttribute("key_filter");
		var keyFilter = typeof keyFilterAttr == "string" && keyFilterAttr == "true";
		
		var geoipFilterAttr = this.getAttribute("geoip_filter");
		var geoipFilter = typeof geoipFilterAttr == "string" && geoipFilterAttr == "true";
		
		return hidden || keyFilter || geoipFilter; 
	};

};


// virtual members

dtxFramework.DTXElement.prototype.setAttribute = function(attrid, value) {
	var attrValue = this.$attributes[attrid];
	var changed =  attrValue != value;
	var self = this;
	if (attrid == "id") {
		this.$id = value;
		this.$framework.$updateIndex(this.$id, this);
	}
	if (dtxFramework.MOZILLA) {
        if (value != null && typeof value == "string" && value.indexOf("chrome://") == 0)
        {
           value = this.framework.loadFromCache(value);
        }
    }
    
	if (attrid == "designable") {
		if(value == "false")
			this.$designable = false;
		else if (value == "true")
			this.$designable = true;
	}
	else if(attrid == "templateid") {
		this.$templateId = value;
	}
	this.$attributes[attrid] = value;
	
	if (attrid == "geoip"){
		if(this.framework.fileExists("geoip.xml")){
			var config_content = this.framework.loadFile("geoip.xml");
			var configXmlDoc = this.framework.loadXML(config_content);
			var geoipNode  = configXmlDoc.getElementsByTagName("GEOIP");
			if(geoipNode.length != 0){
				var geoip = geoipNode[0].firstChild.nodeValue;
				var geoip_value = (value.search(geoip) == -1);
				this.setAttribute("geoip_filter", String(geoip_value));
			}
		}
	}	
	
	//Filters
	if (attrid == "filters"){
		var key_filter = false;

		if (typeof value == "string" && value != ""){
			if(this.framework.$filters.length > 0){
				key_filter = true;
				for(var i=0; i<this.framework.$filters.length && key_filter; i++){
				    var reg = new RegExp("\\b" + this.framework.$filters[i] + "\\b");
				    if (reg.test(value))
						key_filter = false;
				}
			}
		}
		
		this.setAttribute("key_filter", String(key_filter));
	}

	// XSL transform	
	if (!this.$templateApplied &&
		(attrid == "template" || attrid == "feed" || attrid == "cachefile" || attrid == "templateinterval")) {
		var template = this.getAttribute("template");
		var feed 	     = this.getAttribute("feed");
		var cacheFile 	     = this.getAttribute("cachefile");
		var templateInterval = this.getAttribute("templateinterval");

		
		if (typeof template == "string" && template != "" &&
		    typeof feed     == "string" && feed != ""){
			if (feed.indexOf("://") == -1){
				var contents = this.framework.loadChromeFile(feed);
				this.$applyTemplate(template, contents);
			}
		}
		
		
		if (typeof template == "string" && template != "" &&
		    typeof feed     == "string" && feed != "" &&
		    typeof templateInterval == "string" && templateInterval != "") {
				if (typeof cacheFile == "string" && cacheFile != ""){
						if (templateInterval != ""){
							var downloadTime = this.framework.getLastModifiedTime(cacheFile);
							var d = new Date();
							var curTime = d.getTime();
							var IntervalTime = downloadTime + templateInterval * 60 * 1000 ;
							
							this.$templateApplied = true;
	
							if (this.$forceTemplate || curTime > IntervalTime){
								self.framework.download(feed, 
								function (url, code, contents) {
									if (code == 200) {				
										self.$applyTemplate(template, contents);
										self.framework.saveFile(cacheFile, contents);
									}
								});
							}
							this.$forceTemplate = false;
						}				
						
						var contents = this.framework.loadFile(cacheFile);
							if (contents != null) {
							this.$applyTemplate(template, contents);
							}else {
							this.framework.download(feed, 
								function (url, code, contents) {
									if (code == 200) {				
										self.$applyTemplate(template, contents);
										self.framework.saveFile(cacheFile, contents);
									}
								});
							}
				}
		}

	}
	
	if (this.$domNode != null) {
		if (dtxFramework.IE) {
			//$IE_START$//
			try {
				if (attrid != "title")
					this.$domNode.setAttribute(attrid, value);
				if (attrid == "tooltiptext")
					this.$domNode.setAttribute("title", value);
				if (attrid == "class") {
					if (!this.$designMode)
						this.$domNode.className = value;
				} 
				else if (attrid == "hidden" || attrid == "key_filter" || attrid == "geoip_filter") {
					if (this.isHidden()) {
						if (this.$domNode.style.display != "none"){
							if (dtxFramework.IE && DTFramework.WEB && DTFramework.CHROME )
							    this.$prevDisplay = new String(this.$domNode.ownerDocument.defaultView.getComputedStyle (this.$domNode, null).display);
							 else 
								this.$prevDisplay = this.$domNode.style.display;
							
						}
						this.$domNode.style.display = "none";
					}
					else {
						if (this.$domNode.tagName.toLowerCase() != "td" || this.$domNode.attachEvent)
							this.$domNode.style.display = "block";
						else {
							this.$domNode.style.display = (typeof this.$prevDisplay == "string")?this.$prevDisplay:"table-cell";
						}
					}
				}
				else if (attrid == "style") {
					this.$update();
					//this.$domNode.style.cssText = value;
				}
			}
			catch (e) {}
			//$IE_END$//
		}
		else if (dtxFramework.MOZILLA) {
			//$MOZILLA_START$//
			// avoid setting inline event handlers, otherwise they will be called twice
			if (attrid != "oncommand" && attrid != "onclick")
				if (attrid == "hidden" || attrid == "key_filter" || attrid == "geoip_filter")
					this.$domNode.setAttribute("hidden", String(this.isHidden()));
				else
					this.$domNode.setAttribute(attrid, value);
			//$MOZILLA_END$//
		}
	}
	
	// persistence
	if (changed) {
		var persist = this.getAttribute("persist");
		if (typeof persist == "string") {
			persist = " " + persist + " ";
			if (persist.indexOf(" " + attrid + " ") != -1) {
				var evtObj = {target: self, id: attrid, "value": value};
				this.$fireEvent("attributechange", evtObj);
				this.framework.$persistAttribute(this.$id, attrid, value);
			}
		}
	}
    if(attrid == "track") {
        if (value == "true")
            self.framework.mainContainer.$fireEvent("newStatObject", {target: self.framework.mainContainer, "element": self});
    }
};

dtxFramework.DTXElement.prototype.getAttribute = function(attrid) {
	if (attrid == "id") 
		return this.$id;
	if (attrid == "designable") 
		return this.$designable.toString();
	if(attrid == "templateid")
		return this.$templateId;
	else {
		/*
		if (this.$domNode != null)
			return this.$domNode.getAttribute(attrid);
		else
		*/
		var value = this.$attributes[attrid];
		return value;
        
		/*			
		var value = this.$attributes[attrid];
		if (this.$domNode != null && this.$domNode.hasAttribute(attrid)) {
			value = this.$domNode.getAttribute(attrid);
			this.$attributes[attrid] = value;
		}
		return value;
		*/
	}
};

dtxFramework.DTXElement.prototype.addEventListener = function(eventid, handler) {
	var handlers = this.$events[eventid];
	if (handlers == null) {
		handlers = new Array();
		this.$events[eventid] = handlers;
	}
	var i = 0;
	for (; i < handlers.length && handlers[i] != handler; i++);
	if (i == handlers.length) {
		handlers.push(handler);
	}
};

dtxFramework.DTXElement.prototype.focus = function() {
	if (this.$domNode != null)
		this.$domNode.focus();
};

dtxFramework.DTXElement.prototype.fireEvent = function(eventid, eventObj) {
	this.$fireEvent(eventid, eventObj);
};

dtxFramework.DTXElement.prototype.$fireEvent = function(eventid, eventObj) {
	var handlers = this.$events[eventid];
	if (handlers != null) {
		for (var i = 0; i < handlers.length; i++) {
			// Calling to delegated code can never be trusted,
			//	if one handler fails other handlers should still be called
			try {
				var handler = handlers[i](eventObj);
			}
			catch (e) {
				this.framework.getLogger().error(e.message);
			}
		}
	}
	var inlineHandler = this.getAttribute("on" + eventid);
	if (typeof inlineHandler == "function") {
		var fn = inlineHandler;
		fn.apply(eventObj, []);
	}
};

dtxFramework.DTXElement.prototype.$applyAttributes = function() {
	if (this.$domNode != null) {
		for (var attrid in this.$attributes) {
			this.setAttribute(attrid, this.$attributes[attrid]);
		}
		//this.$domNode.setAttribute(this.framework.applicationId, "true");
	}
};

dtxFramework.DTXElement.prototype.$connectDOM = function() {
	this.$updateId++;
};
	
dtxFramework.DTXElement.prototype.$disconnectDOM = function() {
	this.$domNode = null;
};
	
dtxFramework.DTXElement.prototype.$applyStyle = function() {
};
	
dtxFramework.DTXElement.prototype.$hookEvents = function() {
	if (dtxFramework.IE) {
		//$IE_START$//
		var self = this;
		if (this.$domNode != null) {
			this.$domNode.onmousedown = function(evn) {
				if (!evn)
					evn = self.$domNode.document.parentWindow.event;
				var x = evn.x;
				var y = evn.y;
				if (typeof x == "undefined" || typeof y == "undefined") {
					x = evn.layerX;
					y = evn.layerY;
				}
				var eventObj = {target: self, cancelBubble: false, "x": x, "y": y};
				self.$fireEvent("mousedown", eventObj);
				if (eventObj.cancelBubble)
					evn.cancelBubble = true;
			};
			this.$domNode.onmouseup = function(evn) {
				var eventObj = {target: self, cancelBubble: false};
				self.$fireEvent("mouseup", eventObj);
				if (!evn)
					evn = self.$domNode.document.parentWindow.event;
				if (eventObj.cancelBubble)
					evn.cancelBubble = true;
				
				// fire onclick here to avoid first click being lost to focus window
				eventObj = {target: self, cancelBubble: false};
				self.$fireEvent("click", eventObj);
				if (eventObj.cancelBubble)
					evn.cancelBubble = true;
			};
			this.$domNode.ondblclick = function(evn) {
				var eventObj = {target: self, cancelBubble: false};
				self.$fireEvent("dblclick", eventObj);
				if (!evn)
					evn = self.$domNode.document.parentWindow.event;
				if (eventObj.cancelBubble)
					evn.cancelBubble = true;
			};
			this.framework.addEventListener(this.$domNode, "mouseover", function(evn) {
				var eventObj = {target: self, cancelBubble: false};
				self.$fireEvent("mouseover", eventObj);
				if (!evn)
					evn = self.$domNode.document.parentWindow.event;
				if (eventObj.cancelBubble)
					evn.cancelBubble = true;
			});
			this.framework.addEventListener(this.$domNode, "mouseout", function(evn) {
				var eventObj = {target: self, cancelBubble: false};
				self.$fireEvent("mouseout", eventObj);
				if (!evn)
					evn = self.$domNode.document.parentWindow.event;
				if (eventObj.cancelBubble)
					evn.cancelBubble = true;
			});
		}
		//$IE_END$//
	}
};
	
dtxFramework.DTXElement.prototype.$setParent = function(node) {
	this.parentNode = node;
	if (this.framework.mainContainer != null && !this.$notifiedCreation) {
		this.framework.mainContainer.$fireEvent("newelement", {target:this.framework.mainContainer, element:this});
		this.$notifiedCreation = true;
	}
};

dtxFramework.DTXElement.prototype.$createDOMNode = function() {
	return null;
};

dtxFramework.DTXElement.prototype.$setDesignMode = function(on) {
	this.$designMode = on;
};

dtxFramework.DTXElement.prototype.$renderOptions = function(node) {
	var label = this.$framework.createElement("label");
	label.setAttribute("value", "No options for this item");
	node.appendChild(label);
};

dtxFramework.DTXElement.prototype.localeAttributesToStr = function(){
	return this.localeAttributesToArray().join(" ");
};

dtxFramework.DTXElement.prototype.localeAttributesToArray = function(asString) {
	var localize = this.getAttribute('localize');
	if (localize){
		var indexes = localize.split(" ");
		for (var i=0 ; i<indexes.length ; i++){
			var idx = indexes[i];
			var attr = String(idx ).split(":")[0];
			var index = String(idx ).split(":")[1];
			if (typeof index == "string"){
				this.$elementIndexes[attr] = index ;
				//localize = "";
			}
		}
	}

	
	var customlocalAttributes = (!this.framework.isEmptyString(localize))?(new String(localize).match(/(\w+)/g)):[];
	
	return this.$localeAttributes.concat(customlocalAttributes);
};
