dtxFramework.EventSink = function(framework) {
	
	this.$framework = framework;
	var self = this;
	
    if (dtxFramework.MOZILLA) {
		//$MOZILLA_START$//

        this.$doc = null ;
        this.$location = "";
        this.$contentWindow = null;
        this.$toolbarLoaded = new Array();
        
        addEventListener("DOMContentLoaded", function(event) {
          self.$doc = event.originalTarget;
          if (self.$doc == null) return;
          self.$location = self.$doc.location;
          self.$contentWindow = self.$doc.defaultView;
        }, false);

		this.$mozObservers = new function(){
			var observerSelf = this;
			this.$httpChannelObserverService = null;

			this.observe = function(subject, topic, data){
				var src = {topLevelRequest: false, tabBrowser: null, initialRequest: false};
	
				if(topic == "http-on-modify-request"){
					//onBeforeNavigate
					var httpChannel = subject.QueryInterface(Components.interfaces.nsIHttpChannel);
					src.topLevelRequest = httpChannel.loadFlags & httpChannel.LOAD_INITIAL_DOCUMENT_URI;
					src.tabBrowser = this.getBrowserFromChannel(httpChannel);
					src.initialRequest = (httpChannel.originalURI.spec == httpChannel.URI.spec);
					self.$onBeforeNavigate(src, httpChannel.URI.spec, httpChannel);
				}else if (topic == "http-on-examine-response") {
				 	//navigateComplete
				 	var httpChannel = subject.QueryInterface(Components.interfaces.nsIHttpChannel);

				 	src.topLevelRequest = httpChannel.loadFlags & httpChannel.LOAD_INITIAL_DOCUMENT_URI;
				 	src.tabBrowser = this.getBrowserFromChannel(httpChannel);
				 	if (httpChannel.responseStatus >= 400
				 			&& src.topLevelRequest
				 			&& gBrowser.contentWindow == httpChannel.notificationCallbacks.getInterface(Ci.nsIDOMWindow) /* notify concerned window only*/){
				 		self.$onNavigateError(src, httpChannel.URI.spec, httpChannel.responseStatus);
				 	}else{
				 		self.$onNavigateComplete(src, httpChannel.URI.spec);					
				 	}	
				}
			};

			this.getBrowserFromChannel = function (httpChannel){
				try{  
					var notificationCallbacks =   httpChannel.notificationCallbacks;
					if(!notificationCallbacks && httpChannel.loadGroup){
						notificationCallbacks = httpChannel.loadGroup.notificationCallbacks;
					}
					if (notificationCallbacks){
						return gBrowser.getBrowserForDocument(notificationCallbacks.getInterface(Components.interfaces.nsIDOMWindow).top.document); 
					}
				}catch (e) {}
				
				return null;
			};

			this.QueryInterface =  function(aIID) {
					if (aIID.equals(Components.interfaces.nsIWebProgressListener) ||
					    aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
					    aIID.equals(Components.interfaces.nsISupports)){
						return this;
					}
					throw Components.results.NS_NOINTERFACE;
			};

			this.onLocationChange = function (webProgress , request, location) {
				self.$location = location.spec;
				self.$onLocationChange(location.spec);
				return 0;
			};
		
			this.registerListener = function(){
				observerSelf.$httpChannelObserverService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
				observerSelf.$httpChannelObserverService.addObserver(observerSelf,"http-on-examine-response",false);
				observerSelf.$httpChannelObserverService.addObserver(observerSelf,"http-on-modify-request",false);
				observerSelf.$httpChannelObserverService.addObserver(observerSelf,"EndDocumentLoad",false);
				window.getBrowser().addProgressListener(observerSelf);
				window.getBrowser().addEventListener("DOMContentLoaded", function(evt) {
					if(typeof self.$contentWindow.toolbarLoaded === 'undefined' && !self.$contentWindow.toolbarLoaded){
					    self.$contentWindow.toolbarLoaded = new Array();
					}
					if (typeof self.$contentWindow.toolbarLoaded !== 'undefined' && self.$contentWindow.toolbarLoaded.indexOf(framework.applicationId) == -1){    
						if (self.$doc !=  null)
						    self.$onDocumentComplete(self.$doc.location.href);						
						self.$contentWindow.toolbarLoaded.push(framework.applicationId);
					}
				});
			};
		
			this.unregisterListener = function(){
				try{
					observerSelf.$httpChannelObserverService.removeObserver(observerSelf,"http-on-examine-response");
					observerSelf.$httpChannelObserverService.removeObserver(observerSelf,"http-on-modify-request");
					observerSelf.$httpChannelObserverService.removeObserver(observerSelf,"EndDocumentLoad");
					window.getBrowser().removeProgressListener(observerSelf);
					self.$onQuit();
				}catch(e){
				}
			};
		};
		
		window.addEventListener("load",self.$mozObservers.registerListener,false);
		window.addEventListener("unload",self.$mozObservers.unregisterListener,false);
		
		//$MOZILLA_END$//
	};

	this.$getHandlerId = function(element){
		if (element == null || element.$id == null || element.$id == ''){
			return "$framework"+this.$framework.generateId();
		}
		return element.$id;
	};

	/* Dispatch the events to the event handlers */
	this.$onBeforeNavigate = function(src, url, navObject, ieCancel){
		// navObject represents:
		// HttpChannel object for FF
		src.cancel = false;
		src.newUrl = '';
		
		for (var i in self.$eventHandlers.onBeforeNavigateHandlers) {
			try{
				self.$eventHandlers.onBeforeNavigateHandlers[i](src, url, navObject);
			}catch(e){
				//Make sure that other handlers will be called even if one of them throw a exception
				self.$framework.getLogger().error('onBeforeNavigate Handler Error ['+i+']: '+e.message);
			}

			if (src.cancel) {
				navObject.cancel(Components.results.NS_BINDING_ABORTED);
				break;
			}
		}

		if(src.cancel && src.newUrl)
			if(src.tabBrowser)
				src.tabBrowser.loadURI(src.newUrl);	//SG src.tabBrowser is null for Firefox 38 with e10s. 
			else
				loadURI(src.newUrl);
	};
	
	this.$onNavigateComplete = function(src, url){
		for (var i in self.$eventHandlers.onNavigateCompleteHandlers) {
			try{
				self.$eventHandlers.onNavigateCompleteHandlers[i](src, url);
			}catch(e){
				//Make sure that other handlers will be called even if one of them throw a exception
				self.$framework.getLogger().error('onNavigateComplete Handler Error ['+i+']: '+e.message);
			}
		}
	};
	
	this.$onQuit = function(){
		for (var i in self.$eventHandlers.onQuitHandlers) {
			try{
				self.$eventHandlers.onQuitHandlers[i]();
			}catch(e){
				//Make sure that other handlers will be called even if one of them throw a exception
				self.$framework.getLogger().error('onQuit Handler Error ['+i+']: '+e.message);
			}
		}
	};
	
	this.$onLocationChange = function(url){
		for (var i in self.$eventHandlers.onLocationChangeHandlers) {
			try{
				self.$eventHandlers.onLocationChangeHandlers[i](url);
			}catch(e){
				//Make sure that other handlers will be called even if one of them throw a exception
				self.$framework.getLogger().error('onLocationChange Handler Error ['+i+']: '+e.message);
			}
		}
	};

	this.$onDocumentComplete = function(url, disp){
		for (var i in self.$eventHandlers.onDocumentCompleteHandlers) {
			try{
				self.$eventHandlers.onDocumentCompleteHandlers[i](url, disp);
			}catch(e){
				//Make sure that other handlers will be called even if one of them throw a exception
				self.$framework.getLogger().error('onDocumentComplete Handler Error ['+i+']: '+e.message);
			}
		}
	};
	
	this.$onNavigateError = function(src, url, statusCode){
		for (var i in self.$eventHandlers.onNavigateErrorHandlers) {
			try{
				self.$eventHandlers.onNavigateErrorHandlers[i](src, url, statusCode);
			}catch(e){
				//Make sure that other handlers will be called even if one of them throw a exception
				self.$framework.getLogger().error('onNavigateError Handler Error ['+i+']: '+e.message);
			}
		}
	};
	
	this.$onTabActivation = function(tabActivated){
		for (var i in self.$eventHandlers.onTabActivationHandlers) {
			try{
				self.$eventHandlers.onTabActivationHandlers[i](tabActivated);
			}catch(e){
				self.$framework.getLogger().error('onTabActivation Handler Error ['+i+']: '+e.message);
			}
		}
	};
	
	this.addOnBeforeNavigateHandler = function(element, handler) {
		this.$eventHandlers.onBeforeNavigateHandlers[this.$getHandlerId(element)] = handler;
	};
	
	this.addOnNavigateCompleteHandler = function(element, handler) {
		this.$eventHandlers.onNavigateCompleteHandlers[this.$getHandlerId(element)] = handler;
	};
	
	this.addOnQuitHandler = function(element, handler) {
		this.$eventHandlers.onQuitHandlers[this.$getHandlerId(element)] = handler;
	};
	
	this.addOnDocumentCompleteHandler = function(element, handler) {
		this.$eventHandlers.onDocumentCompleteHandlers[this.$getHandlerId(element)] = handler;
	};
	
	this.addonLocationChangeHandler = function(element, handler) {
		this.$eventHandlers.onLocationChangeHandlers[this.$getHandlerId(element)] = handler;
	};
	
	this.addOnNavigateErrorHandler = function(element, handler) {
		this.$eventHandlers.onNavigateErrorHandlers[this.$getHandlerId(element)] = handler;
	};
	
	this.addOnTabActivationHandler = function(element, handler) {
		this.$eventHandlers.onTabActivationHandlers[this.$getHandlerId(element)] = handler;
	};

	this.$eventHandlers = {
		onBeforeNavigateHandlers: new Array(),
		
		onNavigateCompleteHandlers: new Array(),
		
		onQuitHandlers: new Array(),
		
		onDocumentCompleteHandlers: new Array(),
		
		onLocationChangeHandlers: new Array(),
		
		onNavigateErrorHandlers: new Array(),
		
		onTabActivationHandlers: new Array()
	};
};