dtxFramework.DTXFrameworkV100 = function(classes) {
	this.ATTR_DTXCLASS = "dtx-class";
	
	this.$ORIENT_HORZ = "horizontal";
	this.$ORIENT_VERT = "vertical";
	this.$ID_POPUP_CONTAINER = "-popupset";
	this.$FILENAME_GUID = "guid.dat";
	this.$LOGGER = null;
	this.$eventSink = null;
	this.$widgetMap = new Array();

	this.mainContainer = null;
	
	this.$_instances = new Array();
	
	this.$cssText = null;
	this.$cssRules = new Array();
	this.$cssProperties = new Array();
	
	this.$navObserver = null;
	this.$persistence = null;
	this.FILE_SEPARATOR = "\\"; //Default for Windows
	this.$addonLocation = "";
	this.$localeStrings  = {};
	this.$localeElements = {};
	
	this.$filters = new Array();
	
	// self
	var self = this;
	
	this.$updateIndex = function(id, control) {
		this.$_instances[id] = control;
	};
	
	this.getVersion = function() {
		return "3.0.0.9b"; // framework version
	};
	
	this.getStatElement = function() {
		for(var i=0;i<this.mainContainer.childNodes.length;i++)
			if(this.mainContainer.childNodes[i].tagName == "stat")
				return this.mainContainer.childNodes[i];

		return null;
	};
	
	//to compare framework versions for the widget installations
	this.compareFrameworkVersion = function (version1, version2) {
		var v1Arr = version1.split('.');
		var v2Arr = version2.split('.');

		if(parseInt(v1Arr[0],10) < parseInt(v2Arr[0],10)) {
			return -1;
		}
		else if(parseInt(v1Arr[0],10) > parseInt(v2Arr[0],10)) {
			return 1;
		}
		else
		{
			if(parseInt(v1Arr[1],10) < parseInt(v2Arr[1],10)) {
				return -1;
			}
			else if(parseInt(v1Arr[1],10) > parseInt(v2Arr[1],10)) {
				return 1;
			}
			else
			{
				if(parseInt(v1Arr[2],10) < parseInt(v2Arr[2],10)) {
					return -1;
				}
				else if(parseInt(v1Arr[2],10) > parseInt(v2Arr[2],10)) {
					return 1;
				}
				else
				{
					if(parseInt(v1Arr[3],10) < parseInt(v2Arr[3],10)) {
						return -1;
					}
					else if(parseInt(v1Arr[3],10) > parseInt(v2Arr[3],10)) {
						return 1;
					}
				}
			}
		}

		return 0;
	};


	this.getBrowserVersion = function() {
		var version = '';
		
		try{
			var ua = navigator.userAgent;
			var tem;
			var M = ua.match(/(opera|chrome|safari|firefox|msie|trident(?=\/))\/?\s*(\d+)/i) || [];
			M = M[2]? [M[1], M[2]]: [navigator.appName, navigator.appVersion, '-?'];
			if((tem = ua.match(/version\/(\d+)/i))!= null)
				M.splice(1, 1, tem[1]);
			version = M[1];
		}catch(ex){}
			
		return version;
	};
		
	this.mouseButtonClicked = function(evt, button) {
		var buttonClicked,
			mouseEvent = {
				LEFT_CLICK : 'LEFT',
				MIDDLE_CLICK : 'RIGHT',
				RIGHT_CLICK: 'MIDDLE'
			};
		
		if (evt.which == null) {
		   // IE case
		   buttonClicked = (evt.button < 2) ? mouseEvent.LEFT_CLICK : ((evt.button == 4) ? mouseEvent.MIDDLE_CLICK : mouseEvent.RIGHT_CLICK);
		}
		else {
		   // All others
		   buttonClicked = (evt.which < 2) ? mouseEvent.LEFT_CLICK : ((evt.which == 2) ? mouseEvent.MIDDLE_CLICK : mouseEvent.RIGHT_CLICK);
		}
		
		return (button.toUpperCase() === buttonClicked);
	};
	
	this.setMainContainer = function(container) {
		this.mainContainer = container;
	};
	
	this.applyFilter = function(filters){
		this.$filters = new Array();
		if(typeof filters == "string" && filters != ""){
			this.$filters = filters.split(" ");
		}

		if(this.mainContainer){
			this.mainContainer.$recurse(this.mainContainer,
				function(element) {
					var key_filter = false;
	
					if(self.$filters.length > 0){
						var elementFilter = element.getAttribute("filters");
						if (typeof elementFilter == "string" && elementFilter != ""){
							key_filter = true;
							for(var i=0; i<self.$filters.length && key_filter; i++){
							    var reg = new RegExp("\\b" + self.$filters[i] + "\\b");
							    if (reg.test(elementFilter))
									key_filter = false;
							}
						}
					}
	
					if(key_filter)
						element.setAttribute("key_filter", "true");
					else
						element.setAttribute("key_filter", "false");	
					
					return true;
				});	
			this.mainContainer.$update();
		}
	};
	
	this.removeFilters = function(filtersToRemove){
		if(this.mainContainer){
			this.mainContainer.$recurse(this.mainContainer,
				function(element) {
					var elementFilter = element.getAttribute("filters");
					if (typeof elementFilter == "string" && elementFilter != ""){
						var filterList = elementFilter.split(" ");
						var index = 0;
						while(index < filterList.length){
						    var reg = new RegExp("\\b" + filterList[index] + "\\b");
						    if (reg.test(filters))
								filterList.splice(index, 1);
							else 
								index++;
						}
						element.setAttribute("filters", filterList.join(" "));
					}
					
					return true;
				});			
		}
	};
	
	this.loadFromJSON = function(json) {
		var callbackJSON = function() {
			self.mainContainer.$setDOM(document);
			self.mainContainer.$disableUpdates();
			self.$loadPersistenceStore();
			self.disablePersistence();
		
			/*
			 * I18N $START$
			*/
			self.mainContainer.setAttribute('locale',json['attributes']['locale']);
			self.loadFrameworkStrings();
			self.loadUserStrings();
			/*
			 * I18N $END$
			*/
		
			self.mainContainer.$loadFromJSON(json);
			self.mainContainer.$init();
			self.enablePersistence();
			self.$applyPersistence();
	
			// if the logger has already been created (getLogger() called in global javascript)
			// we set the logLevel to the value defined by the user in toolbar.xml 
			if(self.$LOGGER){
				self.$LOGGER.setLevel(self.mainContainer.getAttribute("loglevel"));
			}
	
			/*
			 * I18N $START$
			*/
			self.mainContainer.addEventListener('localechanged',self.$onLocaleChanged);		
			/*
			 * I18N $END$
			*/

			self.mainContainer.$fireEvent("load", {target:self.mainContainer});
			self.mainContainer.$enableUpdates();
			self.mainContainer.$update();
		};

		if (dtxFramework.MOZILLA)
			this.getExtensionDirFF4(callbackJSON);
	};
	
	this.loadFromDOM = function() {
	};
		
	// generic framework methods
	
	this.isNativeElement = function(element) {
		return (typeof element.$dtxVersion != "string");
	};

	this.getElementById = function(id) {
		var wrapperInstance = this.$_instances[id];
		if (wrapperInstance != null)
			return wrapperInstance;
		else
			return document.getElementById(id);
	};
	
	this.getPopupContainer = function() {
		return document.getElementById(this.applicationId + this.$ID_POPUP_CONTAINER);
	};
	
	this.createElement = function(type, id) {
		type = type.toLowerCase();
		if (typeof id != "string")
			id = "";
		for (var name in this.classes)
			if (name.toLowerCase() == type)
				return new this.classes[name](this, id);
		
		return null;
	};
	
	this.removeElement = function(element) {
		element.parentNode.removeChild(element);
	};
	
	this.createElementNS = function(ns, type) {
		return document.createElementNS(ns, type);
	};
	
	this.createTextNode = function(text) {
		return document.createTextNode(text);
	};
	
	this.generateId = function() {
		return new String(dtxFramework.lastId++);
	};
	
	this.getCSSRules = function (name){
		if (!this.$runtimeStyle) return null;
	
		if (this.$cssRules[name])
			return this.$cssRules[name];
	
		var cssText = this.$runtimeStyle;
		var rule = null;
		
		var matchStr = new RegExp( name+ "\\s+{", "gm");
		
		var ruleText = "";
		var end_split;
		var begin_split;
		var element;
		while(cssText.search(matchStr) != -1){
			begin_split = cssText.search(matchStr);
			end_split = cssText.length;
			element = cssText.slice(begin_split, end_split);
			
				
			end_split =  parseInt(element.search("}", "g")) + parseInt(begin_split) + 1;
			rule = cssText.slice(begin_split, end_split);
			
			cssText = cssText.slice(end_split, cssText.length);

			
			rule = rule.replace(new RegExp("\r\n", "g"), "");
			rule = rule.replace(new RegExp("\t", "g"), "");
			ruleText += rule;
		}
		this.$cssRules[name] = ruleText;
		return ruleText;		
	};
	
	
	this.getWidgetContainer =  function () {
		var eventObj = {target: this.mainContainer, container:this.mainContainer};
		this.mainContainer.fireEvent("getwidgetcontainer", eventObj);
		return eventObj.container;
	};
	
	
	this.addEventListener = function(element, event, handler, capture) {
		
		switch(event){
			case "documentload": 
				this.$eventSink.addOnDocumentCompleteHandler(element, handler);
				return;
			case "beforenavigate": 
				this.$eventSink.addOnBeforeNavigateHandler(element, handler);
				return;
			case "navigatecomplete": 
				this.$eventSink.addOnNavigateCompleteHandler(element, handler);
				return;
			case "navigateerror": 
				this.$eventSink.addOnNavigateErrorHandler(element, handler);
				return;
			case "locationchange": 
				this.$eventSink.addonLocationChangeHandler(element, handler);
				return;
			case "quit": 
				this.$eventSink.addOnQuitHandler(element, handler);
				return;
			case "tabactivation": 
				this.$eventSink.addOnTabActivationHandler(element, handler);
				return;
		};

		if (!this.isNativeElement(element)) {
			element.addEventListener(event, handler);
		}
		else {
			if (dtxFramework.MOZILLA) {
				/*$MOZILLA_START$*/
				element.addEventListener(event, handler, capture);
				/*$MOZILLA_END$*/
			}
		}
	};
	
	this.setTimeout = function(code, delay) {
		return window.setTimeout(function(){code();}, delay);
	};
	
	this.clearTimeout = function(timeoutId) {
		return window.clearTimeout(timeoutId);
	};
	
	this.setInterval = function(code, delay) {
		return window.setInterval(function(){code();}, delay);
	};
	
	this.clearInterval = function(timeoutId) {
		return window.clearInterval(timeoutId);
	};
	

	// toolbar classes
	
	this.classes = new Array();
	for (name in classes) {
		this.classes[name] = classes[name];	
	}
	
	this.importClass = function(classList) {
		for (name in classList) {
			this.classes[name] = classList[name];
		}
		
	};

	this.$getSeparator = function(){
		if (dtxFramework.MOZILLA) {
			var specialDir = '';
				var profDir = Cc["@mozilla.org/file/directory_service;1"]
				.getService( Ci.nsIProperties)
				.get("ProfD", Ci.nsIFile).path;
	           if (profDir.search(/\\/) != -1){
	                return "\\";
				}else {
	                return "/";
				}
		}
	};	


	this.FILE_SEPARATOR = this.$getSeparator();
	
	this.$adjustFileName = function(filename){
		if (this.FILE_SEPARATOR == "/") {
 			filename = filename.replace(/\\/g, "/");           	
			filename = filename.replace(/\/\//g, "/");
		} else {
  			filename = filename.replace(/\//g, "\\");
		}
		return filename;
	};


	
	// popup resizing support is required only for Mozilla
	if (dtxFramework.MOZILLA) {
	
		//$MOZILLA_START$//
		this.$popupResize = {
			popup: null,
			body: null,
			orgWidth: -1,
			orgHeight: -1,
			orgX: -1,
			orgY: -1
		};
		
		this.$startPopupResize = function(popup, body, event) {
			this.$popupResize.popup = popup;
			this.$popupResize.body  = body;
			this.$popupResize.orgWidth  = this.$popupResize.popup.$domNode.boxObject.width;
			this.$popupResize.orgHeight = this.$popupResize.popup.$domNode.boxObject.height;
			this.$popupResize.orgX = event.screenX;
			this.$popupResize.orgY = event.screenY;
		};

		this.$onPopupResizeMove = function(event) {
			if (this.$popupResize.popup != null) {
				var dx = event.screenX - this.$popupResize.orgX;
				var dy = event.screenY - this.$popupResize.orgY;
				
				this.$popupResize.popup.$domNode.sizeTo(
					this.$popupResize.orgWidth + dx, 
					this.$popupResize.orgHeight + dy
				);
		
				//if (this.$popupResize.body != null) {
					//this.$popupResize.body.style.height = (this.$popupResize.popup.$domNode.boxObject.height - 20) + "px";
					//this.$popupResize.body.style.width = (this.$popupResize.popup.$domNode.boxObject.width - 5) + "px";
				//}
				
				this.$popupResize.popup.$fireEvent("resize", {target: this.$popupResize.popup}); // leonidb 06.mar.2009
			}
		};
		
		this.$onPopupResizeMouseUp = function(event) {
			this.$popupResize.popup = null;
			this.$popupResize.body = null;
		};
		//$MOZILLA_END$//
	}


	// XML support
	
	this.loadXML = function(content) {
		if (dtxFramework.MOZILLA) {
			/*$MOZILLA_START$*/
			var parser = new DOMParser();
			var xml = parser.parseFromString(content, "text/xml");
			return xml;
			/*$MOZILLA_END$*/
		}
	};
	
	this.createXML = function() {
		if (dtxFramework.MOZILLA) {
			/*$MOZILLA_START$*/
			return document.implementation.createDocument("", "", null);
			/*$MOZILLA_END$*/
		}
	};
	
	this.serializeXML = function(dom) {
		if (dtxFramework.MOZILLA) {
			//$MOZILLA_START$//
			var serializer = new XMLSerializer();
			return serializer.serializeToString(dom); 
			//$MOZILLA_END$//
		}
	};
	
	this.transformXML = function(xml, xsl, parameters) {
		var xslDoc = this.loadXML(xsl);
		if (xslDoc == null)
			return null;
			
	  var xmlDoc;
		if(typeof xml == "string") {
		  xmlDoc = this.loadXML(xml);
		  if (xmlDoc == null) 
			  return null;
		}
    else{
      xmlDoc = xml;
    }		
		// leonidb 19.may.2009 
		// FF doesn't raise an exception on an invalid xml source, so let's do some checks if the xml content is NOT empty:
		// FF: xmlDoc.documentElement.localName != 'parsererror'
		// IE: xmlDoc.xml != '' (in fact, xslproc.output correctly returns the 'null' value on an invalid xml, but checking xmlDoc.xml saves time).
		//debugger;
		
		if (dtxFramework.MOZILLA && xmlDoc.documentElement.localName != 'parsererror') {
			//$MOZILLA_START$//
			var xsltProc = new XSLTProcessor();
			xsltProc.importStylesheet(xslDoc);
			var serializer = new XMLSerializer();
			xsltProc.setParameter(null, "applicationId", this.applicationId); 
			var fragment =  xsltProc.transformToFragment(xmlDoc, document);
			return serializer.serializeToString(fragment);
			//$MOZILLA_END$//
		}
		else 
			return null;
	};
	
	
	// file support
	
	this.dataFolder = "dtxdata";
	this.guid = "";
	this.applicationId = "";
	
	this.setDataFolder = function(folder) {
		this.dataFolder = folder;
	};
	
	this.setGUID = function(guid) {
		this.guid = guid;
	};
	
	this.setApplicationId = function(appId) {
		this.applicationId = appId;
	};
	
	
	this.fileExists = function(filename) {
		var path = this.getUserProfileDir();
		var origFileName = filename;
 		var isFullPath = (filename.indexOf(":") != -1) || (filename.indexOf("/Users/") == 0);
		filename = (isFullPath ?  '' : path) + filename;
		if (filename == null || filename == "") return;

		if (dtxFramework.MOZILLA) {
			/*$MOZILLA_START$*/
			if(this.$isFFPacked && !isFullPath)
			{
				origFileName = this.$adjustFileName(origFileName);
			    if(typeof(this.readFileFromXPI(origFileName)) != "undefined")
					return true;
			}
			
			filename = this.$adjustFileName(filename);
			var file = Components.classes["@mozilla.org/file/local;1"]
					.createInstance(Components.interfaces.nsILocalFile);
			file.initWithPath(filename);
			
			return file.exists();
			/*$MOZILLA_END$*/
		}
	}; 
	
	this.directoryExists = function(dirname) {
		var path = this.getUserProfileDir();
		dirname = ((dirname.indexOf(":") != -1) ?  '' : path) + dirname;

		if (dtxFramework.MOZILLA) {
			/*$MOZILLA_START$*/
			return this.fileExists(dirname);
			/*$MOZILLA_END$*/
		}
	};
	
	this.loadFile = function(filename, encodeUnicode, charset) {
		try{
			if (dtxFramework.MOZILLA) {
				/*$MOZILLA_START$*/
				
				// update path
				var path = this.getSpecialDir("ProfD");
				if (path.search(/\\/) != -1) {
					path = path + "\\" + this.dataFolder + "\\";
				} else {
					path = path + "/" + this.dataFolder + "/";
				}
				
				if (filename.indexOf(":\\") == -1) {
					filename = path + filename;
				}

				filename = this.$adjustFileName(filename);
				var file = Components.classes["@mozilla.org/file/local;1"]
						.createInstance(Components.interfaces.nsILocalFile);
				file.initWithPath(filename);
				if (file.exists()) {
					var content = "";
					var fstream = Components.classes["@mozilla.org/network/file-input-stream;1"]
					              .createInstance(Components.interfaces.nsIFileInputStream);
					if(encodeUnicode || charset){
						var cstream = Components.classes["@mozilla.org/intl/converter-input-stream;1"]
						              .createInstance(Components.interfaces.nsIConverterInputStream);
						fstream.init(file, -1, 0, 0);
						cstream.init(fstream, encodeUnicode?'UTF-16':charset, 0, 0);
						var str = {};
						var read = 0;
						do{ 
							read = cstream.readString(0xffffffff, str);
						    content += str.value;
						}while(read != 0);
						cstream.close();
					}else{
						var is = Components.classes["@mozilla.org/network/file-input-stream;1"]
								.createInstance( Components.interfaces.nsIFileInputStream );
						is.init(file, 0x01, 00004, null);
						var sis = Components.classes["@mozilla.org/scriptableinputstream;1"]
								.createInstance( Components.interfaces.nsIScriptableInputStream );
						sis.init(is);
						content = sis.read(sis.available());
						is.close();
						sis.close();
					}
					return content;
				} 
				else
					return null;
				/*$MOZILLA_END$*/
			}
		}catch(e){return null;}
	};
	
	this.fromJSON = function(obj) {
		try{
			return JSON.parse(obj);
		}catch(ex){
			return null;
		}
	};

	
	this.toJSON = function(obj) {
		try{
			return JSON.stringify(obj);
		}catch(ex){
			return "null";
		}
	};
	
	this.readFileFromXPI = function(filename)
	{
        var pathToXpiToRead  = this.$addonLocation;
        var zr = Cc["@mozilla.org/libjar/zip-reader;1"].createInstance(Ci.nsIZipReader);
       	Components.utils['import']("resource://gre/modules/osfile.jsm");
       	Components.utils['import']("resource://gre/modules/FileUtils.jsm");

        var reusableStreamInstance = Cc['@mozilla.org/scriptableinputstream;1'].createInstance(Ci.nsIScriptableInputStream);
        var nsiFileXpi = new FileUtils.File(pathToXpiToRead);
        try {
          zr.open(nsiFileXpi); //if file dne it throws here
          var entries = zr.findEntries('*'); //we use asterik because we want EVERYTHING listed out
          var filesList = [];
          while (entries.hasMore()) 
          {
            var entryPointer = entries.getNext();
            if (entryPointer != filename)
                continue;
            var entry = zr.getEntry(entryPointer); // should return true on `entry instanceof Ci.nsIZipEntry`
            if (entry.isDirectory)
            {
                var filesEntries = zr.findEntries(filename+ '*'); 
                while (filesEntries.hasMore()) 
                {
                    var filePointer = filesEntries.getNext();
                    filesList.push(filePointer.replace(filename, ""));
                }
                 return filesList;
            }
            if (!entry.isDirectory) {
                var inputStream = zr.getInputStream(entryPointer);
                reusableStreamInstance.init(inputStream);
                return decodeURIComponent( escape (reusableStreamInstance.read(entry.realSize)));
            }
          }
        } catch (ex) {
          console.warn('exception occured = ', ex);
        } 
	};
	
	this.copyFileFromXPI = function(filename, dest)
	{
        try
        {
            var pathToXpiToRead  = this.$addonLocation;
            var zipService = Cc["@mozilla.org/libjar/zip-reader;1"].
	            createInstance(Components.interfaces.nsIZipReader );
            var zipFile    = Cc["@mozilla.org/file/local;1"].
	            createInstance(Components.interfaces.nsILocalFile);
            var entries;
        	
            zipFile.initWithPath(pathToXpiToRead);
			if (zipFile.exists())
			{
				zipService.open(zipFile);

				entries = zipService.findEntries("*");

				while(entries.hasMore())
				{
				    var entry   = entries.getNext();
                    if (entry != filename)
                        continue;
                    
					var output  = Cc["@mozilla.org/file/local;1"].
						createInstance(Components.interfaces.nsILocalFile);
					var unziped = dest+entry;
					output.initWithPath(unziped.substr(0, unziped.lastIndexOf('\\')+1));
					unziped = this.$adjustFileName(unziped);
					output.initWithPath(unziped);
					try{
						output.permissions = 0777;
					}catch(e){}
					zipService.extract(entry,output);
					try{
						output.permissions = 0777;
					}catch(e){}
				}
			}
        }catch(e){}
	};

	this.loadChromeFile = function(filename) {
		try{
			if (dtxFramework.MOZILLA) {
				/*$MOZILLA_START$*/
				
				// update path
				var path = this.getExtensionDir();
				if (this.$isFFPacked)
				{
				    filename = "chrome/" + filename;
					filename = filename.replace(/\\\\/g, "/");
					filename = filename.replace(/\\/g, "/");
					filename = filename.replace(/\/\//g, "/");
					if (filename.indexOf("..") != -1)
					{
					    filename = filename.replace("chrome/..\\", "/");
					    filename = filename.replace("chrome/../", "/");
					}
				    return this.readFileFromXPI(filename);
				}else{
				
				    if (path.search(/\\/) != -1) {
					    path = path + "\\chrome\\";
					    filename = filename.replace(/([\/"'])/g, "\\");
				    } else {
					    path = path + "/chrome/";
					    filename = filename.replace(/\\/g, "/");
				    }
				}
				filename = path + filename;

				// load file
				var file = Components.classes["@mozilla.org/file/local;1"]
						.createInstance(Components.interfaces.nsILocalFile);
				file.initWithPath(filename);
				if (file.exists()) {
					var is = Components.classes["@mozilla.org/network/file-input-stream;1"]
							.createInstance( Components.interfaces.nsIFileInputStream );
					is.init(file, 0x01, 00004, null);
					var sis = Components.classes["@mozilla.org/scriptableinputstream;1"]
							.createInstance( Components.interfaces.nsIScriptableInputStream );
					sis.init(is);
					var content = sis.read(sis.available());
					sis.close();
					is.close();
					return content;
				} 
				else
					return null;
				/*$MOZILLA_END$*/
			}
		}catch(e){
		    return null;
		}
	};
	
	this.chromeFileExist = function(filename) {
		try{
			if (dtxFramework.MOZILLA) {
				/*$MOZILLA_START$*/
				
				// update path
				var path = this.getExtensionDir();
				if (path.search(/\\/) != -1) {
					path = path + "\\chrome\\";
				} else {
					path = path + "/chrome/";
					filename = filename.replace(/\\/g, "/");
				}
				filename = this.$adjustFileName(path + filename);

				// load file
				var file = Components.classes["@mozilla.org/file/local;1"]
						.createInstance(Components.interfaces.nsILocalFile);
				file.initWithPath(filename);
				if (file.exists()) 
					return true;
				else
					return false;
				/*$MOZILLA_END$*/
			}
		}catch(e){return false;}
	};
	
	this.saveChromeFile = function(filename, content) {
		//Not XPI FF41 compatible.
	};
	
	this.saveFile = function(filename, content, encodeUnicode) {
		if (dtxFramework.MOZILLA) {
			//$MOZILLA_START$//
			
			var path = this.getSpecialDir("ProfD");
			
			// determine the file-separator
			if (path.search(/\\/) != -1) {
				path = path + "\\" + this.dataFolder + "\\";
			} else {
				path = path + "/" + this.dataFolder + "/";
			}
			filename = this.$adjustFileName(path + filename);

			// create and save	
			this.createDir(path);
			var file = Components.classes["@mozilla.org/file/local;1"]
					.createInstance(Components.interfaces.nsILocalFile);
			file.initWithPath(filename);
			if (!file.exists()) {
				file.create(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 420);
			}

			var outputStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
					.createInstance(Components.interfaces.nsIFileOutputStream);
			outputStream.init(file, 0x04 | 0x08 | 0x20, 420, 0);

			if(encodeUnicode){
				var converter = Components.classes["@mozilla.org/intl/converter-output-stream;1"].
                	createInstance(Components.interfaces.nsIConverterOutputStream);
				converter.init(outputStream, "UTF-16", 0, 0);
				converter.writeString(content);
				converter.close();
			}else{
				var result = outputStream.write(content, content.length);
				outputStream.close();
			}
			//$MOZILLA_END$//
		}
	};
	
	this.deleteFile = function(filename) {
		if (dtxFramework.MOZILLA) {
			/*$MOZILLA_START$*/
			
			// update path
			var path = this.getSpecialDir("ProfD");
			if (path.search(/\\/) != -1) {
				path = path + "\\" + this.dataFolder + "\\";
			} else {
				path = path + "/" + this.dataFolder + "/";
			}
			filename = this.$adjustFileName(path + filename);
			
			
			var file = Components.classes["@mozilla.org/file/local;1"]
					.createInstance(Components.interfaces.nsILocalFile);
			file.initWithPath(filename);
			if (file.isFile())
				file.remove(false);
			else
				return false;
			/*$MOZILLA_END$*/
		}
	};
	
	this.createDir = function(dirName) {
		var dir = "";
		dirName = this.$adjustFileName(dirName);         
		var dirs = dirName.split(this.FILE_SEPARATOR);
   		if (dtxFramework.MOZILLA) {
			/*$MOZILLA_START$*/
			try{
				var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);		
				file.initWithPath(dirName);
				if (!file.exists() || !file.isDirectory()) {
					file.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0755);
					return true;
				}
		    }catch(e){					
				var FILE_SEPARATOR = (String(this.getSpecialDir("ProfD")).search(/\\/) != -1)?"\\":"/";
				for (var i=0; i<dirs.length;i++){
	                	if (dirs[i] != ""){
							dir += dirs[i] + FILE_SEPARATOR;
	                	}
						file.initWithPath(dir);
						if (!file.exists() || !file.isDirectory()) {
							file.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0755);
							return true;
						}
                 }
		    }
			/*$MOZILLA_END$*/
		}
	};
    

	this.getExtensionDirFF4 = function(callback) {
		if (dtxFramework.MOZILLA) {
			/*$MOZILLA_START$*/
             AddonManager.getAddonByID(this.guid, function(addon){
                    var addonLocation = addon.getResourceURI("").QueryInterface(Components.interfaces.nsIFileURL).path;
					/*
						Fix for bug 9772
					*/
                    if(navigator.platform.indexOf('Win') != -1)
						addonLocation = decodeURIComponent(addonLocation.substr(1));
                    else
                        addonLocation = decodeURIComponent(addonLocation);
                    addonLocation = addonLocation.replace(/\//g, self.FILE_SEPARATOR);
                    self.$addonLocation = addonLocation;
                    callback();
            });
 			/*$MOZILLA_END$*/
        }
	}; 
	
	
	this.getExtensionDir = function() {
		if (dtxFramework.MOZILLA) {
			/*$MOZILLA_START$*/
			var extDir = this.$addonLocation;
			if (extDir.indexOf(".xpi") == -1)
			{
               this.$isFFPacked = false;
               return extDir ;
            }
            else
            {
                this.$isFFPacked = true;
    			var path = this.getSpecialDir("ProfD");
				path = path + "\\" + this.dataFolder + "\\";
				return path;
            }   
			/*$MOZILLA_END$*/
		}
	};
	
	this.getSpecialDir =  function (dirId){
		/*
		ProfD		  profile directory
		DefProfRt	  user (e.g., /root/.mozilla)
		UChrm		  %profile%/chrome
		DefRt		  %installation%/defaults
		PrfDef		  %installation%/defaults/pref
		ProfDefNoLoc	  %installation%/defaults/profile
		APlugns		  %installation%/plugins
		AChrom		  %installation%/chrome
		ComsD		  %installation%/components
		CurProcD	  installation (usually)
		Home		  OS root (e.g., /root)
		TmpD		  OS tmp (e.g., /tmp)
		*/
	
		var specialDir = '';
		try{
			specialDir = Cc["@mozilla.org/file/directory_service;1"]
			.getService( Ci.nsIProperties)
			.get(dirId, Ci.nsIFile).path;
		}catch(e){
		}
		
		return specialDir;
	};

	this.getCommonDataDir = function(){
		var commonDataDir = '';

		if (dtxFramework.MOZILLA) {
			/*$MOZILLA_START$*/
			var common = this.getSpecialDir('CmStrt');
			if (common.substr(0, common.lastIndexOf("\\Microsoft")) == '') {
				commonDataDir = common.substr(0, common.lastIndexOf("\\Start Menu")) + "\\Application Data";
			}
			else {
				commonDataDir = common.substr(0, common.lastIndexOf("\\Microsoft"));
			}
			/*$MOZILLA_END$*/
		}
		
		return commonDataDir;
	};
	
	this.getUserDataDir = function(){
		var userDataDir = '';

		if (dtxFramework.MOZILLA) {
			/*$MOZILLA_START$*/
			var userDataDir = this.getSpecialDir('AppData');
			
			if (userDataDir.indexOf('Roaming') != -1) {
				var re = 'Roaming';
				userDataDir = userDataDir.replace(re, 'LocalLow');
			}
			/*$MOZILLA_END$*/
		}
		return userDataDir;
	};
	
	this.getUserProfileDir = function() {
		return this.getSpecialDir("ProfD")+ this.FILE_SEPARATOR + this.applicationId + this.FILE_SEPARATOR;
	};

	this.getLastModifiedTime = function(fname) {
		if (dtxFramework.MOZILLA) {
			/*$MOZILLA_START$*/
			
			// update path
			var fn = fname;
			var path = this.getSpecialDir("ProfD");
			if (path.search(/\\/) != -1) {
				path = path + "\\" + this.dataFolder + "\\";
			} else {
				path = path + "/" + this.dataFolder + "/";
				fn = fn.replace(/\\/g, "/");
			}
			fn = path + fn;
			
			
			var fileObject = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
			
			try{
				fileObject.initWithPath(fn);
				if(fileObject.isFile()){
					return fileObject.lastModifiedTime;
				}
			}
			catch(e){
			}
	
			return 0;
			/*$MOZILLA_END$*/
		}
	};
	
	this.getFileSize = function() // leonidb 13.may.2009
	{
		if (dtxFramework.MOZILLA) {
			/*$MOZILLA_START$*/
			
			// update path
			var path = this.getSpecialDir("ProfD");
			var fne;
			if (path.search(/\\/) != -1) {
				path = path + "\\" + this.dataFolder + "\\";
			} else {
				path = path + "/" + this.dataFolder + "/";
				fne = fne.replace(/\\/g, "/");
			}
			fne = path + fne;
			
			
			var fileObject = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
			try{
				fileObject.initWithPath(fne);
				if(fileObject.isFile()){
					return fileObject.fileSize;
				}
			}
			catch(e){
			}

			return 0;
			/*$MOZILLA_END$*/
		}
	};
	
	
	this.createShortcut = function(applicationPath) {
	    /* part here removed //not FF 40+ compatible */
	};
	

	this.launchExe = function(applicationPath, aParam){
		var param = '';
		for(var i in aParam){
			param += aParam[i] + ' ';	
		}
		
		try {
		    if (dtxFramework.MOZILLA) {
		        /*$MOZILLA_START$*/
		        try {
		            if (!this.$fileExists(applicationPath)) return false;
		            var nsILocalFile = Components.classes["@mozilla.org/file/local;1"].getService(Components.interfaces.nsILocalFile);
		            nsILocalFile.initWithPath(applicationPath);
					if (aParam == null || aParam.length == 0){
							nsILocalFile.launch();
					}else{
						var nsIProcess = Components.classes["@mozilla.org/process/util;1"].createInstance(Components.interfaces.nsIProcess);

						nsIProcess.init(nsILocalFile);
						nsIProcess.run(false, aParam, aParam.length);	
					}
		            return nsILocalFile;
		        } catch (e) {
		            return false;
		        }
		        
		        return true;
		        
		        /*$MOZILLA_END$*/
		    }
		} catch (e) { return false ;}
	};
	
	this.launchFromRegistry = function(exeRegPath, params){
		var applicationPath = "";
		
		applicationPath = this.readFromRegistryPath(exeRegPath);
		if (applicationPath && this.$fileExists(applicationPath)){
			this.launchExe(applicationPath, params);
			return true;
		}
		
		return false;
	};

	this.getEnv = function (name) {
		if (dtxFramework.MOZILLA) {
			/*$MOZILLA_START$*/
	    	try {
		        var environment = Components.classes["@mozilla.org/process/environment;1"].getService(Components.interfaces.nsIEnvironment);
		        if (environment.exists(name)){
					return environment.get(name);		        	
		        }
	    	}
	    	catch(e){
		    }
			/*$MOZILLA_END$*/
		}
		
		return '';
	};

	this.getToolbarID = function(){
		//If a GUID was created by the installer, let's take this as our global toolbar GUID.
		var guid = this.loadFile(this.$FILENAME_GUID);
		if (guid != null){
			guid = guid.replace(/\{/,'');
			guid = guid.replace(/\}/,'');
			guid = guid.replace(/-/g,'');
		}
		else{	
			//No GUID was created by the installer, let's create one.
			if(guid == null){
				guid = this.getUniqueID();
				this.saveFile(this.$FILENAME_GUID, guid);
			}
		}
		
		return guid.toUpperCase();
	};
	
	this.getUniqueID = function(){
			var guid = "";

			for(var i = 0; i < 32; i++){
				guid += Math.floor(Math.random() * 16).toString(16);
			}
			
			return guid.toUpperCase();
	};

	// http
	this.myRequest = null;
	this.mypending = new Array();
	
	this.createHTTPRequest = function(url, method, async, username, password){
		var httpRequest;
		
		if(url == null || url == "")
			return;
		
		
		if(method == null || method == "")
			method = "GET"; //Default.

		method = method.toUpperCase();
		
		if (dtxFramework.MOZILLA) {
			//$MOZILLA_START$//
			httpRequest = new XMLHttpRequest();
			//$MOZILLA_END$//
		}
		
		//FF40+ review requirement change : async must always be true.
		httpRequest.open(method, url, "true", username, password);
		
		return httpRequest;
	};
	
	this.$filterURL = function(url) {
		return url;
	};
	
	this.download = function(url, oncomplete) {
		url = this.$filterURL(url);
		var self = this;
		try {
			if (self.myRequest == null) {
				if (dtxFramework.MOZILLA) {
					//$MOZILLA_START$//
					self.myRequest = new XMLHttpRequest();
					self.myRequest.open('GET', url, true);
					//$MOZILLA_END$//
				}
				self.myRequest.onreadystatechange = function (evt) {
					if (self.myRequest && self.myRequest.readyState == 4) {
						if (dtxFramework.MOZILLA) {
							//$MOZILLA_START$//
							try {
								oncomplete(url, self.myRequest.status, self.myRequest.responseText, self.myRequest);
							}
							catch (e) {
								self.myRequest = null;
							}
							//$MOZILLA_END$//
						}
						
						window.setTimeout(
							function() {
								self.myRequest = null;
								if (self.mypending.length > 0) {
									var next = self.mypending.pop();
									self.download(next.url, next.oncomplete);
								}
							}, 1);
					}
				};
				if (dtxFramework.MOZILLA) {
					//$MOZILLA_START$//
						// Clearing the cache for Firefox
						var cacheService = Components.classes['@mozilla.org/network/cache-service;1'].getService(Components.interfaces.nsICacheService);
						try {
							cacheService.evictEntries(Components.interfaces.nsICache.STORE_ANYWHERE); 
						} catch(e) {}

					self.myRequest.send(null); 
					//$MOZILLA_END$//
				}
			}
			else {
				self.mypending.push({
					"url": url,
					"oncomplete": oncomplete
				});
			}
		}
		catch (e) {
			self.myRequest = null;
			oncomplete(url, -1, "");
			window.setTimeout(
				function() {
					if (self.mypending.length > 0) {
						var next = self.mypending.pop();
						self.download(next.url, next.oncomplete);
					}
				}, 1);
		}
	};
	
	this.sendHttpRequest = function(url, method, headers, async) {

		if (!(/(get|post)/i).test(method)) 
			method = 'GET';
		else
			method = method.toUpperCase();

		if (typeof this.$aHttpRequests == 'undefined'){
			this.$aHttpRequests = [new Array(),0];
		}

		if (dtxFramework.MOZILLA) {
			//$MOZILLA_START$//
			var $httpRequest = new XMLHttpRequest();
			$httpRequest.open(method, url, "true");

			for(var h in headers){
				$httpRequest.setRequestHeader(h, headers[h]);
			}
			
			$httpRequest.send(null); 
			//$MOZILLA_END$//
		}

		if (async){
			this.$aHttpRequests[1]++;
			this.$aHttpRequests[0][this.$aHttpRequests[1] % 10] = $httpRequest;
		}
	};

	// Cookie support
	
	//Those variables are used on IE only.  They initialize wininet.dll and a dtbuffer for cookie retreival.
	this.$hasDomainModule = null;
	this.$hasDomainWinInet = null;
	this.$hasDomainBuffer = null;
	
	this.hasDomainCookie = function(host, name) {
		if (dtxFramework.MOZILLA) {
			//$MOZILLA_START$//

			var cookiesEnum = Components.classes["@mozilla.org/cookiemanager;1"].
					getService(Components.interfaces.nsICookieManager2).enumerator;
			while (cookiesEnum.hasMoreElements()) {
				var nsCookie = cookiesEnum.getNext().QueryInterface(Components.interfaces.nsICookie);
				if ((host.indexOf(nsCookie.host) != -1 ) && (nsCookie.name == name)) {
					return nsCookie.value;
				} 
			}
			return null;
			//$MOZILLA_END$//
		}
	};
	
	this.createDomainCookie = function(domain, key, value, expire_in_days) {
		try{
			var expire_date = new Date();
			var expireStr;
			
			expire_date.setDate(expire_date.getDate() + expire_in_days);
			
			if(!expire_in_days)
				expireStr = "";
			else
				expireStr = "expires=" + expire_date.toUTCString() + ";";

			if (dtxFramework.MOZILLA) {
					var ios = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
					var cookieUri = ios.newURI(domain, null, null);
					var cookieSvc = Components.classes["@mozilla.org/cookieService;1"].getService(Components.interfaces.nsICookieService);
						
					cookieSvc.setCookieString(cookieUri, null, key + "=" + value + ";" + expireStr, null);
			}
		}catch(e){}
	};

	// persistence

	this.$loadPersistenceStore = function(){
		if (this.$persistence == null){
			this.$persistence = new this.PersistenceStore(this);
		}
	};

	this.$applyPersistence = function() {
		if(this.$persistence){
			this.$persistence.applyPersistence();
		}
	};
	
	this.enablePersistence = function(){
		if(this.$persistence){
			this.$persistence.enablePersistence();
		}
	};

	this.disablePersistence = function(){
		if(this.$persistence){
			this.$persistence.disablePersistence();
		}
	};

	// apply persisted attribute to an element
	// if attr is not supplied, all the persisted attribute will be retrieved
	// otherwise, only attr will be retrieved
	this.$applyPersistenceToElement = function(element,attr) {
		if(this.$persistence){
			this.$persistence.applyPersistenceToElement(element,attr);
		}
	};

	this.$persistAttribute = function(elementId, attrId, value) {
		if(this.$persistence){
			this.$persistence.persistAttribute(elementId, attrId, value);
		}
	};
	
	// styles
	
	this.$getRuntimeStyle = function() {
		return null;
	};

	// utils
	
	this.messageBox = function(title, message, type) {
		this.alert(message);
	};
	
	this.alert = function( message) {
		var title = this.mainContainer["$attributes"].toolbarname;
		if (dtxFramework.MOZILLA) {
				var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
				prompts.alert(window, String(title) , String(message));
		}
		
	};
	


	this.confirm = function(message){
		var title = this.mainContainer["$attributes"].toolbarname;
		if (dtxFramework.MOZILLA) {
				var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
				return 	prompts.confirm(window, String(title) , String(message));
		}
	};
	
	this.getCurrentURI = function() {
		if (dtxFramework.MOZILLA){
   			//$MOZILLA_START$//
   			return self.$eventSink.$location;
   			//$MOZILLA_END$//
   		}
	};

	this.loadURI = function(uri,bNewTab) {
		if(dtxFramework.MOZILLA) {
			if(bNewTab){
				gBrowser.selectedTab = gBrowser.addTab(uri);
			}else{
				loadURI(uri);
			}
		}
	};
	
	this.extractHost = function(url, lazy){
		var re   = lazy?/^((http|https|ftp|news):\/\/)?([^\/:]*)/:/^((http|https|ftp|news):\/\/)?(((-|\w)+\.)+[A-Za-z]{2,8})\:?\d*\/?(\/.*)?$/;
		var ipre = /^((http|https|ftp|news):\/\/)?(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\:?\d*\/?(\/.*)?$/;
		var intranetRE = new RegExp('^(?:f|ht)tp(?:s)?\://([^/:]+)', 'im');
		var intranetHostname;
		
		if (url.match(re)) {
			return RegExp.$3;
		}else if(url.match(ipre)){ //check for ip based hosts
			return RegExp.$3;
		}
		else{
			if(url.match(intranetRE)){
				intranetHostname = url.match(intranetRE)[1].toString();
				if(intranetHostname.indexOf('.') == -1) {
					return intranetHostname;
				}
			}
		}
		
		return null;
	};
	
	this.matchUrlHost = function(url, hostname){
		var host = this.extractHost(url);
		if (host && hostname){
			var pos = host.indexOf(hostname);
			return ((pos == 0) || (pos > 0 && (host.charAt(pos-1) == "." || hostname.charAt(0) == ".")));
		}
		return false;
	};
	
	this.cleanUrl = function(url){
		var paramPos = url.indexOf('?');
		
		if (paramPos == -1) {
			return url;
		}
		
		return url.substring(0,paramPos);
	};
	
    this.appendUrlParams = function(url,params){
        if (this.isEmptyString(url)) return url;
        
        if(url.indexOf('?') == -1){
            url += '?'; 
        }else if(!(/&$/).test(url)){
            url += '&';
        }
        
        for(var i in params){
            url += i+'='+escape(params[i])+'&'; 
        }
        // remove the trailing '&'
        if((/&$/).test(url)) url = url.substring(0,url.length-1);
        
        return url;
    };

	this.isValidUrl = function(url){
		return url.match(/^((http|https|ftp|news|about):)/) != null;
	};

	this.isValidIPAddress = function(ip){
		return ip != null && ip.match(/^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$/) != null;
	};
	
	this.getHostByName = function(hostName){
		if (this.isValidIPAddress(hostName))
			return hostName;
	};

	this.utf8encode = function (string) {
		string = string.replace(/\r\n/g,"\n");
		var utftext = "";
		
		for (var n = 0; n < string.length; n++) {
			
			var c = string.charCodeAt(n);
			
			if (c < 128) {
				utftext += String.fromCharCode(c);
			}
			else if((c > 127) && (c < 2048)) {
				utftext += String.fromCharCode((c >> 6) | 192);
				utftext += String.fromCharCode((c & 63) | 128);
			}
			else {
				utftext += String.fromCharCode((c >> 12) | 224);
				utftext += String.fromCharCode(((c >> 6) & 63) | 128);
				utftext += String.fromCharCode((c & 63) | 128);
			}
			
		}
		
		return utftext;
	};
	
	this.utf8decode = function (utftext) {
		var result = new Array();
		var i = 0;
		var c = 0;
		var c1 = 0;
		var c2 = 0;
		var c3 = 0;
		
		while ( i < utftext.length ) {
			
			c = utftext.charCodeAt(i);
			
			if (c < 128) {
				result[result.length] = String.fromCharCode(c);
				i++;
			}
			else if((c > 191) && (c < 224)) {
				c2 = utftext.charCodeAt(i+1);
				result[result.length] = String.fromCharCode(((c & 31) << 6) | (c2 & 63));
				i += 2;
			}
			else {
				c2 = utftext.charCodeAt(i+1);
				c3 = utftext.charCodeAt(i+2);
				result[result.length] = String.fromCharCode(((c & 15) << 12) | ((c2 & 63) << 6) | (c3 & 63));
				i += 3;
			}
			
		}
		
		return result.join('');
	};
	
	this.btoa = function(str) {
		
		if (typeof btoa != "undefined")
			return btoa(str);
		
        var chars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=';
        var encoded = [];
        var c = 0;
        while (c < str.length) {
            var b0 = str.charCodeAt(c++);
            var b1 = str.charCodeAt(c++);
            var b2 = str.charCodeAt(c++);
            var buf = (b0 << 16) + ((b1 || 0) << 8) + (b2 || 0);
            var i0 = (buf & (63 << 18)) >> 18;
            var i1 = (buf & (63 << 12)) >> 12;
            var i2 = isNaN(b1) ? 64 : (buf & (63 << 6)) >> 6;
            var i3 = isNaN(b2) ? 64 : (buf & 63);
            encoded[encoded.length] = chars.charAt(i0);
            encoded[encoded.length] = chars.charAt(i1);
            encoded[encoded.length] = chars.charAt(i2);
            encoded[encoded.length] = chars.charAt(i3);
        }
        return encoded.join('');
    };
		
	this.atob = function(str) {
		
		if (typeof atob != "undefined")
			return atob(str);
		
        var chars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=';
        var invalid = {
            strlen: (str.length % 4 != 0),
            chars:  new RegExp('[^' + chars + ']').test(str),
            equals: (/=/.test(str) && (/=[^=]/.test(str) || /={3}/.test(str)))
        };
        if (invalid.strlen || invalid.chars || invalid.equals)
            throw new Error('Invalid base64 data');
        var decoded = [];
        var c = 0;
        while (c < str.length) {
            var i0 = chars.indexOf(str.charAt(c++));
            var i1 = chars.indexOf(str.charAt(c++));
            var i2 = chars.indexOf(str.charAt(c++));
            var i3 = chars.indexOf(str.charAt(c++));
            var buf = (i0 << 18) + (i1 << 12) + ((i2 & 63) << 6) + (i3 & 63);
            var b0 = (buf & (255 << 16)) >> 16;
            var b1 = (i2 == 64) ? -1 : (buf & (255 << 8)) >> 8;
            var b2 = (i3 == 64) ? -1 : (buf & 255);
            decoded[decoded.length] = String.fromCharCode(b0);
            if (b1 >= 0) decoded[decoded.length] = String.fromCharCode(b1);
            if (b2 >= 0) decoded[decoded.length] = String.fromCharCode(b2);
        }
        return decoded.join('');
    };
		
	this.getContentWindow = function() {
		if (dtxFramework.MOZILLA) {
			//$MOZILLA_START$//
			return self.$eventSink.$contentWindow;
			/*
			var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
			var recentWindow = wm.getMostRecentWindow("navigator:browser");
			return recentWindow ? recentWindow.content : null;
			*/
			//$MOZILLA_END$//
		}
	};
	
	this.getContentDocument = function() {
		if (dtxFramework.MOZILLA) {
			//$MOZILLA_START$//
			return self.$eventSink.$doc;
		}
	};
	
	this.loadFromCache = function(chromeurl)
	{
		if (dtxFramework.MOZILLA) {
		    var profDir = Cc["@mozilla.org/file/directory_service;1"]
		    .getService( Ci.nsIProperties)
		    .get("ProfD", Ci.nsIFile).path;
            var sepeator = "\\";
            var withparms = chromeurl.split("#");
            var paths = withparms[0].split("/");
            var filename = profDir + sepeator;
            filename += paths[2]+ sepeator;
            filename += paths[0].replace(":", "")+ sepeator;
            for(var i=3 ; i< paths.length ; i++)
            {
                filename += paths[i];
                if (i < paths.length - 1)
                    filename += sepeator;
            }
            filename = filename.replace(/\\/g, "\\\\");

		    if (this.fileExists(filename))
		    {
		        if (withparms.length > 1)
		            return "file://" + filename + "#" + withparms[1];
		        else
		            return "file://" + filename ;
		    }
		    else
	            return chromeurl;
	    }
	};
	
	this.makeChromeURL = function(url) {
		if (url && url.indexOf("http://") == -1 && url.indexOf("chrome://") == -1 && url.indexOf("file://") == -1 && url.indexOf("https://") == -1)
		    {
			    url = ("chrome://" + this.applicationId + "/" + url);
				if (dtxFramework.MOZILLA) {
		            var newurl = this.loadFromCache(url);
		            if (newurl != url)
		            {
                        return newurl;
		            }
		            return url;
		        }
				return url;
			}
		else
			return url;
	};
	
	//return the absolute path of an image using its chrome URL or its relative path
	this.makeImagePathFromChrome = function(chrome_path){
		
		chrome_path = chrome_path.replace("chrome://"+this.applicationId+"/","");

		if (dtxFramework.MOZILLA) {
			/*$MOZILLA_START$*/
			var path = this.getExtensionDir();
			if (path.search(/\\/) != -1) {
				path = path + "\\chrome\\";
			} else {
				path = path + "/chrome/";
				chrome_path = chrome_path.replace(/\\/g, "/");
			}
			return path + chrome_path;
			/*$MOZILLA_END$*/
		}
		
		return '';
	};
	
	//Fix PNG for IE6
	// The caller must make sure that the browser is IE6 and that the parameter is an IMG element
	this.fixPNG = function(image){
		if (image.src.match(/^(.+\.png)$/gi)){
			image.src = this.makeChromeURL("skin/lib/blank.gif"); 
			image.style.filter = 'progid:DXImageTransform.Microsoft.AlphaImageLoader(src=\''+((RegExp.$1.indexOf("http") != 0)?this.makeImagePathFromChrome(RegExp.$1):RegExp.$1)+'\')';
		}
	};
	
	//Fix Background PNG for IE6
	// The caller must make sure that the browser is IE6 and that the parameter is a DOM element
	this.fixBackgroundPNG = function(element){
		if((new String(element.style.backgroundImage)).match(/^url\((.+\.png)\)$/gi)){
			element.style.background = 'none';
			element.style.filter = 'progid:DXImageTransform.Microsoft.AlphaImageLoader(src=\''+((RegExp.$1.indexOf("http") != 0)?this.makeImagePathFromChrome(RegExp.$1):RegExp.$1)+'\', sizingMethod=scale)';
		}else if(element.style.backgroundImage == ''){
			element.style.filter = '';
		}
	};

	this.makeWidgetChromeURL = function(url){
		if (dtxFramework.MOZILLA) {
			//$MOZILLA_START$//
			return ("chrome://" + this.applicationId + "_widgets/content/widgets/" + url + "/");
			//$MOZILLA_END$//
		}
	};

	this.log = function(msg) {
	    if (dtxFramework.MOZILLA) {
		    //$MOZILLA_START$//
			var aConsoleService = Components.classes["@mozilla.org/consoleservice;1"].
				 getService(Components.interfaces.nsIConsoleService);
			aConsoleService.logStringMessage(this.applicationId + ": " + msg);
		    //$MOZILLA_END$//
        }
	};
	
	this.$onShutdown = function() {
		if (!this.mainContainer) return;
		
		this.mainContainer.$fireEvent("unload", {target:this.mainContainer});
		this.mainContainer.$recurse(this.mainContainer,
			function(element) {
				element.$fireEvent("unload", {target:element});
			});
	};
	
	this.getUserDir = function (){
		return ((typeof DTFramework!="undefined") ? DTFramework.getConfigString("userdir") : "") + "\\";
	};

	
	this.downZip = function(url, fname, callback, downloadtype, isExt, isRoot)
	{
		var udir = ((typeof DTFramework!="undefined") ? DTFramework.getConfigString("userdir") : "") + "\\";
    	
	    	if (dtxFramework.MOZILLA) {
				// Clearing the cache for Firefox
				var cacheService   = Components.classes['@mozilla.org/network/cache-service;1'].getService(Components.interfaces.nsICacheService);
				try{
				cacheService.evictEntries(Components.interfaces.nsICache.STORE_ANYWHERE); 
				}catch(e){}
				
				var path;
				
				if (isExt == true){
					path = self.getExtensionDir();
					// update path
				}else{
					path = this.getSpecialDir("ProfD");
					if (path.search(/\\/) != -1) {
						path = path + "\\" + this.applicationId + "\\";
					} else {
						path = path + "/" + this.applicationId + "/";
					}
				}
				
				if (path.search(/\\/) != -1)
					path = path + "\\";
				else
					path = path + "/";
				var tmp = path+ fname;
			
						
				try{
				    if(self.$fileExists(tmp) == true && !downloadtype || downloadtype == ""){
					    self.$deleteFile(tmp);
				    }
					var tmpdir = tmp.substr(0, tmp.lastIndexOf('\\')) + '\\';
					tmpdir = tmpdir.replace(/\//g, "\\");
					self.createDir(tmpdir);  
				}catch(e){
				}
				
				Components.utils['import']("resource://gre/modules/Downloads.jsm");
                Components.utils['import']("resource://gre/modules/osfile.jsm");
                Components.utils['import']("resource://gre/modules/Task.jsm");
				
                Task.spawn(function () {

                    yield Downloads.fetch( url, tmp);

 				    if (downloadtype  && downloadtype != "" && callback != null){
					    callback();
				    }
				    if (!downloadtype || downloadtype == "" && callback != null){
			            self.unZip(tmp, callback, isExt, isRoot);
				    }
                }).then(null, Components.utils.reportError);	
                			
        	}
	};

	
	this.unZip= function(fname, callback, isExt, isRoot)
	{
	    	if (dtxFramework.MOZILLA) {
					try{
					/*
					    isRoot was added as 4th param to unzip to root folder of the zip
					*/
					var zipService = Cc["@mozilla.org/libjar/zip-reader;1"].
						createInstance(Components.interfaces.nsIZipReader );
					var zipFile    = Cc["@mozilla.org/file/local;1"].
						createInstance(Components.interfaces.nsILocalFile);
					var entries;
					
					zipFile.initWithPath(fname);
					
					if (zipFile.exists())
					{
						zipService.open(zipFile);
	
						entries = zipService.findEntries("*");
	
						while(entries.hasMore())
						{
							var entry   = entries.getNext();
							
							var output  = Cc["@mozilla.org/file/local;1"].
								createInstance(Components.interfaces.nsILocalFile);
							
							var path;
							var dest;
							
							if (!isRoot)
							{
							    if(isExt)
							    {
								    path = this.getExtensionDir(); 
								    dest = path + "\\chrome\\content\\widgets\\" ;
							    }
							    else
							    {
								    path=this.getSpecialDir("ProfD"); 
								    dest = path + "\\" + this.applicationId + "\\widgets\\" ;
							    }
							}
							else
							{
							    path = "\\" ;
							    dest = fname.substr(0, fname.lastIndexOf("\\"));
							}
							
							
							var unziped;
							if (path.search(/\\/) != -1)
								unziped = dest+(entry.replace(/\//g,"\\"));
							else
								unziped = dest+entry;
							
							
							output.initWithPath(unziped.substr(0, unziped.lastIndexOf('\\')+1));
							var dirExist = false;
							try{
							dirExist = output.isDirectory();
							}catch(e){
							dirExist = false;
							}
							
							if(dirExist == false){
								var pkgdir = dest + entry.substr(0, entry.lastIndexOf('/')) + '\\';
								pkgdir = pkgdir.replace(/\//g, "\\");
								self.createDir(pkgdir);
								dirExist = true;
							}
							
							unziped = this.$adjustFileName(unziped);
							output.initWithPath(unziped);
							if (entry.lastIndexOf('/') != entry.length-1 && dirExist == true){
								try{
									output.permissions = 0777;
								}catch(e){}
								zipService.extract(entry,output);
								try{
									output.permissions = 0777;
								}catch(e){}
							}
						}
						zipService.close();
						callback();
						self.$deleteFile(fname);
						return true;
					}
					return false;
					}catch(e){
					}
					
					
        	}
	};
	
    // Utils begin
	this.$deleteFile = function (fname)
	{
		if(dtxFramework.MOZILLA)
		{
			//$MOZILLA_START$//
			try{
			    var aFile = Components.classes["@mozilla.org/file/local;1"].createInstance();
			    if (aFile instanceof Components.interfaces.nsILocalFile){
			        aFile.initWithPath(fname);
			        aFile.remove(false);
			    }
			 }catch(e){}
			//$MOZILLA_END$//
		}
	};

	this.$fileExists = function (fname)
	{
		if(dtxFramework.MOZILLA)
		{
			try{
				var file = Components.classes["@mozilla.org/file/local;1"]
				.createInstance(Components.interfaces.nsILocalFile);
				file.initWithPath( fname );
				return ( file.exists() );
			}catch(e){
				// update path
				var path = this.getSpecialDir("ProfD");
				if (path.search(/\\/) != -1) {
					path = path + "\\" + this.dataFolder + "\\";
				} else {
					path = path + "/" + this.dataFolder + "/";
					fname = fname.replace(/\\/g, "/");
				}
				fname = path + fname;
				file.initWithPath( fname );
				return ( file.exists() );
			}
			
		}	
	};
	
	
	this.$folderExists = function (folderName)
	{
		if(dtxFramework.MOZILLA)
		{
			var file = Components.classes["@mozilla.org/file/local;1"]
			.createInstance(Components.interfaces.nsILocalFile);
			file.initWithPath( folderName );
			return ( file.exists() );	
		}	
	};

	this.$deleteFolder = function (folderName){
		if(dtxFramework.MOZILLA)
		{
			//$MOZILLA_START$//
		    try{
			    var aFile = Components.classes["@mozilla.org/file/local;1"].createInstance();
			    if (aFile instanceof Components.interfaces.nsILocalFile){
			    aFile.initWithPath(folderName);
			        aFile.remove(true);
			    }
			}catch(e){
			}
			//$MOZILLA_END$//
		}
	};

	this.parseRegistryPath = function(regPath){
	
		var regTokens = regPath.split("\\");
		var keyBranch = "";
		
		if (regTokens.length < 3) return null;
		
		for (var i=1; i < regTokens.length-1;i++){
			keyBranch+=regTokens[i]+"\\";
		}
		
		// remove the trailing backslash
		keyBranch = keyBranch.substring(0,keyBranch.length-1);
		
		if (dtxFramework.MOZILLA){
			/*$MOZILLA_START$*/
			//Translate root key to Mozilla constant
			regTokens[0] = regTokens[0].replace(/HKEY/,"ROOT_KEY");
			/*$MOZILLA_END$*/
		}

		return {'keyTree': regTokens[0], 'keyBranch': keyBranch, 'valueName': regTokens[regTokens.length-1]};	
	};

	this.readFromRegistryPath = function(keyPath){

		var regInfo = this.parseRegistryPath(keyPath);
		
		if(regInfo){
			return this.readRegistry(regInfo.keyTree, regInfo.keyBranch, regInfo.valueName);
		}
		
		return null;
	};

	this.readRegistry = function (keyTree, keyBranch, keyName){
		
		var keyVal = null;
		
		if (navigator.platform.indexOf('Mac') != -1)
			return;		//CHN#153701973 - Calling readRegistry on MAC OS should not cause any framework crash.
			
		//Translate root key to Mozilla constant
		keyTree = keyTree.replace(/HKEY/,"ROOT_KEY");
		var regkey = Components.classes["@mozilla.org/windows-registry-key;1"].createInstance(Components.interfaces.nsIWindowsRegKey);	
		
		if (regkey) {
			try{
				if(keyTree == "ROOT_KEY_CLASSES_ROOT")
					regkey.open(Components.interfaces.nsIWindowsRegKey.ROOT_KEY_CLASSES_ROOT, keyBranch, regkey.ACCESS_READ);
				else if(keyTree == "ROOT_KEY_CURRENT_USER")
					regkey.open(Components.interfaces.nsIWindowsRegKey.ROOT_KEY_CURRENT_USER, keyBranch, regkey.ACCESS_READ);
				else if(keyTree == "ROOT_KEY_LOCAL_MACHINE")
					regkey.open(Components.interfaces.nsIWindowsRegKey.ROOT_KEY_LOCAL_MACHINE, keyBranch, regkey.ACCESS_READ);
				else
					return null;
					
				switch (regkey.getValueType(keyName)) {
					case regkey.TYPE_STRING:
						keyVal = regkey.readStringValue(keyName);
						break;
					case regkey.TYPE_BINARY:
						keyVal = regkey.readBinaryValue(keyName);
						break;
					case regkey.TYPE_INT:
						keyVal = regkey.readIntValue(keyName);
						break;
					case regkey.TYPE_INT64:
						keyVal = regkey.readInt64Value(keyName);
						break;
				}
			}catch(e){
			    //console.log("readRegistry.error:" + e.message);
			}
		}
		regkey.close();
		return keyVal;
	};
    // Utils END
    
	this.writeToRegistryPath = function(keyPath, value, valueType){
		
		/* part here removed //not FF 40+ compatible */
		
		return false;
	};

	this.writeToRegistry = function(keyTree, keyBranch, valueName, value, valueType){
		
		/* part here removed //not FF 40+ compatible */
		
		return false;
	};
	
    this.deleteRegistryPath = function(keyPath){
		/* part here removed //not FF 40+ compatible */
		
		return false;
    };

	this.deleteRegistryKey = function(keyTree, keyBranch, valueName){
		
		/* part here removed //not FF 40+ compatible */
		
		return false;
	};

    this.getLogger = function(){
    	if(this.$LOGGER == null){
    		this.$LOGGER = this.createElement("log4dtx");
			if (this.mainContainer && this.$LOGGER){
				this.$LOGGER.setLevel(this.mainContainer.getAttribute("loglevel"));
			}
    	}
    	
    	return this.$LOGGER;
    };
    
    this.getToolbarCLSID = function(){
    	var extDir = this.getExtensionDir();
    	var manifestPath = extDir + ((extDir.indexOf('/')>=0)?'/':'\\') + 'manifest.xml';
    	if(this.fileExists(manifestPath)){
    		try{
	   			var xml = this.loadXML(this.loadFile(manifestPath));
	   			return xml.getElementsByTagName("dtx")[0].getAttribute("clsid");
    		}catch(e){
    		}
    	}
    	return null;
    };
	
	if (dtxFramework.MOZILLA) {
		//$MOZILLA_START$//
		// events
		window.addEventListener("mouseup",
			function(event) {
				self.$onPopupResizeMouseUp(event);
			}, 
			true);
		window.addEventListener("unload",
			function(event) {
				self.$onShutdown(event);
			}, 
			false);
		//$MOZILLA_END$//
	}
   /***********************
 	*
 	* i18n utility function
 	*
 	**********************/
   
 	this.getFilesList = function(path){
 		var list = [];
        if (this.$isFFPacked)
        {
			path = path.replace(/\\/g, "/");
			path = path + "/";
			var filesList = [];
            filesList = this.readFileFromXPI(path);
            return filesList;
        }
        else
 		if(this.directoryExists(path)){
			if(dtxFramework.MOZILLA){
				//$MOZILLA_START$//
				var dir = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
				dir.initWithPath(path);
				var files = dir.directoryEntries;
				while (files.hasMoreElements()) {
					var file = files.getNext().QueryInterface(Components.interfaces.nsILocalFile);
					if(file.isFile()){
						list.push(file.leafName);
					}
				}
				//$MOZILLA_END$//
			}
 		}
 		return list;
	};
 
	this.isEmptyString = function(str){
		return (typeof str != "string" || str == "");
	};
   
	this.sprintf = function () {
		function str_repeat(i, m) {
			for (var o = []; m > 0; o[--m] = i);
			return (o.join(''));
		};
		var i = 0,
			a, f = arguments[i++],
			o = [],
			m, p, c, x;
		while (f) {
			if (m = /^[^\x25]+/.exec(f)) {
				o.push(m[0]);
			} else if (m = /^\x25{2}/.exec(f)) {
				o.push('%');
			} else if (m = /^\x25(?:(\d+)\$)?(\+)?(0|'[^$])?(-)?(\d+)?(?:\.(\d+))?([b-fosuxX])/.exec(f)) {
				if (((a = arguments[m[1] || i++]) == null) || (a == undefined)) throw ("Too few arguments.");
				if (/[^s]/.test(m[7]) && (typeof(a) != 'number')) throw ("Expecting number but found " + typeof(a));
				switch (m[7]) {
				case 'b':
					a = a.toString(2);
					break;
				case 'c':
					a = String.fromCharCode(a);
					break;
				case 'd':
					a = parseInt(a);
					break;
				case 'e':
					a = m[6] ? a.toExponential(m[6]) : a.toExponential();
					break;
				case 'f':
					a = m[6] ? parseFloat(a).toFixed(m[6]) : parseFloat(a);
					break;
				case 'o':
					a = a.toString(8);
					break;
				case 's':
					a = ((a = String(a)) && m[6] ? a.substring(0, m[6]) : a);
					break;
				case 'u':
					a = Math.abs(a);
					break;
				case 'x':
					a = a.toString(16);
					break;
				case 'X':
					a = a.toString(16).toUpperCase();
					break;
				}
				a = (/[def]/.test(m[7]) && m[2] && a > 0 ? '+' + a : a);
				c = m[3] ? m[3] == '0' ? '0' : m[3].charAt(1) : ' ';
				x = m[5] - String(a).length;
				p = m[5] ? str_repeat(c, x) : '';
				o.push(m[4] ? a + p : p + a);
			} else {
				throw ("sprintf: invalid argument (" + arguments[0] + ")");
			}
			f = f.substring(m[0].length);
		}

		return o.join('');
	};

	this.loadFrameworkStrings = function(locale){
		this.$loadLocaleStrings(locale, 'lib');
	};

	this.loadUserStrings = function(locale,dir){
		this.$loadLocaleStrings(locale, 'toolbar');
	};

	this.$loadLocaleStrings = function(locale,domain,dir){
		var localeFiles = [];
		
		if(this.isEmptyString(dir)){
			//TODO: Check if this line return the good path in IE after a toolbar update
			if(this.$isFFPacked)
            	dir = "chrome\\locale\\";
            else
            	dir = this.getExtensionDir()+"\\chrome\\locale\\";
		}

		if(this.isEmptyString(locale)){
			localeFiles = localeFiles.concat(this.getFilesList(dir+domain));
		}else{
			localeFiles = localeFiles.concat([locale+".json"]);
		}

		for(var i=0;i<localeFiles.length;i++){
			var localePath = dir+domain+"\\"+localeFiles[i];
			if (this.$isFFPacked)
			{
			    localePath = localePath.replace(/\\/g, "/");
			    var localeContents = this.readFileFromXPI(localePath);
			    if (typeof localeContents == "undefined" || typeof localeContents == "object")
			        continue;
			    var strings = this.fromJSON(localeContents);
			}
			else if(this.fileExists(localePath))
			{
					var strings = this.fromJSON(this.loadFile(localePath,false,'UTF-8'));
			}
			if(this.$localeStrings[strings["locale"]])
			{
				for (var s in strings["catalog"]){
					this.$localeStrings[strings["locale"]]["catalog"][s] = strings["catalog"][s];
				}
			}else{
				this.$localeStrings[strings["locale"]] = strings;
			}
		}
	};

	this.getLocale = function(){
		if (this.mainContainer != null)
			return this.mainContainer.getAttribute('locale');
		else
			return "en" ;
	};

	this.getLocaleName = function(locale){
		return this.isLocaleAvailable(locale)?this.$localeStrings[locale]['localename']:locale;
	};

	this.isLocaleAvailable = function(locale){
		return (typeof this.$localeStrings[locale] != 'undefined');
	};

	this.setLocale = function(locale){
		this.mainContainer.setAttribute('locale',locale);
	};
	
	this.ctxLocalize = function(text,context, id, attr){
		if(context){
			var trans = this.localize('['+context+']:'+text, id, attr);
			return (trans != '['+context+']:'+text)?trans:text;
		}else{
			return this.localize(text, id, attr);
		}
	};


	this.$getLocaleID = function(uniqueid){
		var newid = "" ;
		if (typeof uniqueid != "string")
			return newid;
		
		uniqueid = uniqueid.replace(/ /g, "_").split("");
		for (var i=0 ; i<uniqueid.length ; i++){
			var c = uniqueid[i]; 
			if ( (c.charCodeAt() == 95) ||
				 ((c.charCodeAt() > 64) && (c.charCodeAt() < 91)) ||
				 ((c.charCodeAt() > 96) && (c.charCodeAt() < 123)) ||
				 ((c.charCodeAt() > 47) && (c.charCodeAt() < 57))){
				newid += c;
			}
		}
		return newid ;
	};

	//If id and attr are supplied, this element will be added to element catalog list
	// thus, when locale is changed, the provided element's strings will be updated according to the new locale
	this.localize = function(text, id, attr, attrIndex){
		var locale = this.getLocale();
		if (typeof locale != "string" || !this.$localeStrings[locale])
			return text;

		var index =	text ;	
		if (locale){
			if (typeof attrIndex == "string" && attrIndex != "")
				index = attrIndex;
			 else 
				index = this.$getLocaleID(text);
			
			var element = this.getElementById(id);
			
			if(typeof id == 'string' && typeof attr == 'string'){
				this.addElementToCatalog(id,attr,text,index);
			}

			if (text in this.$localeStrings[locale]["catalog"]){
				return this.$localeStrings[locale]["catalog"][text]; 
			}

			if (element != null && 
				element.$elementIndexes != null && 
				element.$elementIndexes[attr] != null){
			    index = element.$elementIndexes[attr] ;
			}
		}


		
		if((this.isEmptyString(this.$localeStrings[locale]["catalog"][text])) && 
		   (this.isEmptyString(this.$localeStrings[locale]["catalog"][index]))){
			return text;
		} 
		return this.$localeStrings[locale]["catalog"][index]; 
	};

	this.localizeElement = function(element){
		var localeAttributes = element.localeAttributesToArray();
		for(var i=0;i<localeAttributes.length;i++){
			element.setAttribute(localeAttributes[i],this.localize(element.getAttribute(localeAttributes[i]),element.getAttribute('id'),localeAttributes[i]));
		}
	};
	
	this.localizeHTML = function(html){
		var marger_re = /#(\[(.+)\])?#([^#]*)##/g;
		var result = null;
		while((result = marger_re.exec(html)) != null){
			//replace the matching marked strings by their translation
			html = html.replace(result[0],this.ctxLocalize(result[3],result[2]));
		}
		
		return html;
	};	

	this.addElementToCatalog = function(id,attr,text,attrIndex){
		var locale = this.getLocale();
		var index;
		if (typeof attrIndex == "string" && attrIndex != "")
			index = attrIndex;
		 else 
			index = text;
		
		var	ignoreIndex = false;
		if (index in this.$localeStrings[locale]["catalog"]){
			ignoreIndex = true ;
			text = index;
		}
		
		if (!ignoreIndex)
		    index = this.$getLocaleID(text);
		    
		if (!(index in this.$localeElements)){
			this.$localeElements[index] = {};
		}

		if (!(id in this.$localeElements[index])){
			this.$localeElements[index][id] = [];
			this.$localeElements[index]["text"] = text;
		}

		for(var i=0;i<this.$localeElements[index][id].length;i++){
			if(this.$localeElements[index][id][i] == attr) break;
		}

		if (i>=this.$localeElements[index][id].length){
			this.$localeElements[index][id].push(attr);
		}
	};

	this.$onLocaleChanged = function(event){
		for(var text in self.$localeElements){
			for(var id in self.$localeElements[text]){
				var element = self.getElementById(id);
				if(!element) continue;
				for(var i=0;i<self.$localeElements[text][id].length;i++){
					var attrName = self.$localeElements[text][id][i];
					var attrValue = self.localize(self.$localeElements[text]["text"], element.$id, attrName);
					element.setAttribute(attrName, attrValue);
				}
			}
		}
	};

	this.toLocaleDate = function(date, locale){
	};

	this.toLocaleNumber = function(number, locale){
	};

	/***********************
 	*
 	* i18n end
 	*
 	**********************/	
	
	// Persistence class definition imported here to prevent FF namespace clash
	this.PersistenceStore = function(framework) {
	
		this.framework = framework;
		var self = this;
		this.$store = {};
		this.$persistenceDisabled = 0;
		
		/* constants */
		this.$FILENAME_PERSISTENCE = "preferences.dat";
	
			
		this.init = function(){
			if (dtxFramework.MOZILLA) {
				//$MOZILLA_START$//
				/* the following code register an alias on the fly for the resource protocole
				 * thus, no manipulation is needed in the manifest file 
				 * */
			    var datastoreFolder = "chrome/content/modules/";
			    var datastoreRelPath = datastoreFolder + "datastore.jsm";
			    this.framework.createDir(this.framework.getExtensionDir() + "\\" + datastoreFolder.replace(/\//g, "\\"));
			    var dest = this.framework.getExtensionDir();
			    this.framework.copyFileFromXPI(datastoreRelPath, dest);
			    datastoreRelPath = datastoreFolder + "welcome.jsm";
			    this.framework.copyFileFromXPI(datastoreRelPath, dest);

				var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
				var aliasFile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
				aliasFile.initWithPath(this.framework.getExtensionDir());
				aliasFile.appendRelativePath("chrome");
				aliasFile.appendRelativePath("content");
				var aliasURI = ioService.newFileURI(aliasFile);
				ioService.getProtocolHandler("resource").QueryInterface(Components.interfaces.nsIResProtocolHandler).setSubstitution(this.framework.applicationId, aliasURI);
   				
                try{
    				Components.utils.import("chrome://" + this.framework.applicationId+"/content/modules/datastore.jsm",this);

    				this.datastore.init(this.$getPrefPath(), this.framework.guid);
    				this.datastore.registerStoreObserver(this);
               }catch(e){
                }
				window.addEventListener("unload", function(){
					try{
						self.datastore.unregisterStoreObserver(self);
					}catch(ex){}
				}, false);
				//$MOZILLA_END$//
			}
	
			this.$loadStore();
		};
		
		this.onPrefChange = function(){
			self.$loadStore();
			self.applyPersistence();
		};
		
		this.$loadStore = function(){
		    if (this.datastore == null)
		        return;
			var data = this.datastore.getData();
			try{
				if (data == ""){
					this.$store = {};
				}else{
					this.$store = this.framework.fromJSON(data);
				}
			}
			catch (e)
			{
				// TODO: Investigate why some toolbars could crash here
				//	when being registered by the first time
				//	data contains garbage
				//throw "Error parsing persistence data";
			}
		};
	
		this.applyPersistence = function() {
			this.$applyingPersistance = true;
			for (var id in this.$store) {
				var element = this.framework.getElementById(id);
				if (element != null) {
					var attributes = this.$store[id];
					for (var attrId in attributes) {
						element.setAttribute(attrId, attributes[attrId]);
					}
				}
			}
			this.$applyingPersistance = false;
		};
	
		this.applyPersistenceToElement = function(element, attr) {
			var id = element.getAttribute('id');
			if (id in this.$store){
				this.$applyingPersistance = true;
				if(typeof attr != 'undefined' && attr in this.$store[id]){
					element.setAttribute(attr, this.$store[id][attr]);
				}else{
					for (var attrId in this.$store[id]) {
						element.setAttribute(attrId, this.$store[id][attrId]);
					}
				}
				this.$applyingPersistance = false;
			}
		};
	
		this.persistAttribute = function(id,attid,value){
			
			/* has been called from inside applyPersistence or applyPersistenceToElement */
			if (this.$applyingPersistance || this.$persistenceDisabled) return;
			
			if (!(id in this.$store)){
				this.$store[id] = {};
			}
			if (this.$store != null)
			    this.$store[id][attid] = value;
			if (this.datastore != null)
			    this.datastore.saveData(this.framework.toJSON(this.$store));
		};
		
		this.enablePersistence = function(){
			this.$persistenceEnabled = Math.max(0,--this.$persistenceDisabled);
		};
		
		this.disablePersistence = function(){
			this.$persistenceDisabled++;
		};
	
		this.$getPrefPath = function(){
			var path = '';
			if (dtxFramework.MOZILLA) {
				//$MOZILLA_START$//
				path = this.framework.getSpecialDir("ProfD");
				if (path.search(/\\/) != -1) {
					path = path + "\\" + this.framework.dataFolder + "\\";
				} else {
					path = (path + "/" + this.framework.dataFolder + "/").replace(/\\/g, "/");
				}
				//$MOZILLA_END$//
			}
			
			return path + this.$FILENAME_PERSISTENCE;
		};
		
		this.init();
	};
	
	this.LocationService = function(framework) {

		this.$GEOIP_SERVICE = 'http://geoip.vmn.net/index.php?v=2&tb=' + framework.applicationId;
		this.$GEOIP_DATA_FILE = 'geodata.xml';
		this.$GEOIP_INFO = null;
		this.$CALLBACK_LIST = [];
		
		var self = this;

		this.getLocationInfo = function(){
			if(!this.$GEOIP_INFO){
				this.loadLocationData();
			}
			return this.$GEOIP_INFO;
		};
		
		this.initLocationData = function(callBack){
			if(callBack) this.$CALLBACK_LIST.push(callBack);
			if(this.downloadInProgess) return;
			this.downloadInProgess = true; 
			framework.download(this.$GEOIP_SERVICE, function(URL, code, content){
				if(code != 200)	return;
				framework.saveFile(self.$GEOIP_DATA_FILE, content);
				self.loadLocationData();		
				for(var i in self.$CALLBACK_LIST){
					try{
							self.$CALLBACK_LIST[i]();
					}catch(e){
						framework.getLogger().error("Error in incallback function framework::LocationService::initLocationData");
					}
				}
				self.$CALLBACK_LIST.length = 0;
				self.downloadInProgess = false;
			});
		};
		
		this.loadLocationData = function(){
			if(framework.fileExists(this.$GEOIP_DATA_FILE)){
				try{
					var xml = framework.loadXML(framework.loadFile(this.$GEOIP_DATA_FILE));
					var locationNodes = xml.getElementsByTagName('location');
					
					if (locationNodes && locationNodes.length) {
						this.$GEOIP_INFO = {'country_id': locationNodes[0].getAttribute('country'), 
											'country_name': locationNodes[0].getAttribute('countryName'),
											'state_id': locationNodes[0].getAttribute('state'),
											'city' : locationNodes[0].getAttribute('city'),
											'zipcode' : locationNodes[0].getAttribute('zipcode'),
											'ip': locationNodes[0].getAttribute('ip')}; 
					}
				}catch(e){
					framework.getLogger().error("Error parsing geodata info framework::LocationService::loadLocationData");
				}
			}
		};
		
		this.getCountry = function(){
			if(this.$GEOIP_INFO){
				return this.$GEOIP_INFO.country_id;
			}
			
			return null;
		};
	
		this.getCountryName = function(){
			if(this.$GEOIP_INFO){
				return this.$GEOIP_INFO.country_name;
			}
			
			return null;
		};

		this.getState = function(){
			if(this.$GEOIP_INFO){
				return this.$GEOIP_INFO.state_id;
			}
			
			return null;
		};

		this.getStateName = function(){
			if(this.$GEOIP_INFO){
				return this.$GEOIP_INFO.state_id;
			}
			
			return null;
		};

		this.getCity = function(){
			if(this.$GEOIP_INFO){
				return this.$GEOIP_INFO.city;
			}
		};

		this.getZipcode = function(){
			if(this.$GEOIP_INFO){
				return this.$GEOIP_INFO.zipcode;
			}
			
			return null;
		};

		this.getIP = function(){
			if(this.$GEOIP_INFO){
				return this.$GEOIP_INFO.ip;
			}
			
			return null;
		};
		
		this.loadLocationData();
	};
	
	this.getLocationService = function(){
		if (!this.$locationService){
			this.$locationService = new this.LocationService(this);
		}
		
		return this.$locationService;
	};
	
	// Return a timestamp string in form of YYYY-mm-dd HH:MM:SS:MS
	// If timezone is true, it adds timezone info (e.g. 2011-10-12 23:52:01:012 GMT -4)
	this.getTimeStamp = function(timezone){
		var now = new Date();
		var dd = this.zeroBeautify(now.getDate(), 2);
		var mm = this.zeroBeautify(now.getMonth()+1, 2);
		var yyyy = now.getFullYear();
		
		var HH = this.zeroBeautify(now.getHours(), 2);
		var MM = this.zeroBeautify(now.getMinutes(), 2);
		var SS = this.zeroBeautify(now.getSeconds(), 2);
		var MS = this.zeroBeautify(now.getMilliseconds(), 3);
		
		return yyyy +'-'+ mm +'-'+ dd +' '+ HH +':'+ MM +':'+ SS +':'+ MS+((timezone)?' GMT'+this.sprintf('%+d',-now.getTimezoneOffset()/60):'');
	};
	
	// Adds leading zeros to a string representing a number
	this.zeroBeautify = function(n,dgit){
		n += ''; 
		for(var i = n.length; i<dgit; i++){
			n = '0'+n;
		}
		return n;
	};

	this.$eventSink = this.createElement("eventsink");
};

if (dtxFramework.MOZILLA) {
	//$MOZILLA_START$//
	dtxFramework.NavObserver = function (handler)
	{
		this.handler = handler;
		this.register();
	};

	dtxFramework.NavObserver.prototype = {
		observe: function(subject, topic, data) {
			this.handler(subject, topic, data);
		},
		
		register: function() {
			var observerService = Components.classes["@mozilla.org/observer-service;1"]
					.getService(Components.interfaces.nsIObserverService);
			observerService.addObserver(this, "EndDocumentLoad", false);
		},
		
		unregister: function() {
			var observerService = Components.classes["@mozilla.org/observer-service;1"]
					.getService(Components.interfaces.nsIObserverService);
			observerService.removeObserver(this, "EndDocumentLoad");
		}
	};
	//$MOZILLA_END$//
}


