var EXPORTED_SYMBOLS = ["datastore"];

var datastore = {
	/* variable that holde the shared data */
	$data: "",
	
	/* persistence file path*/
	$persistenceFilename: "",
	
	/* Indicates if the data has changed (need to be flushed to the disk) */
	$dirty: false,
	
	/* true if the store is busy saving data to disk */
	$busy: false,
	
	/* flag which indicates if store has been shutdown */
	$halted: false,

	/* timer variables for flushing data to the disk */
	$timer: null,
	$delay: 400, // 400 miliseconds
	
	/* list of data change observers */
	$obslist: [],
	
	/* initialization flag */
	$initialized: false,

	init: function(pFilename, extid){
		if(this.$initialized) return;
		this.$persistenceFilename = pFilename;
		this.$extid = extid;
		this.$uninstall = false;
		
		/* loading data from disk */
		this.$loadData();
		
		/* initializing timer for flushing data */
		this.$timer = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);
		this.$timer.initWithCallback({notify: function(){datastore.$flushData()}},this.$delay,Components.interfaces.nsITimer.TYPE_REPEATING_SLACK);
		
		/* part here removed //not FF 40+ compatible */
		
		try{
			Components.utils.import("resource://gre/modules/AddonManager.jsm");
			AddonManager.addAddonListener(this);
		}catch(e){
			//FF3.6 and down
		}
	},

	getData: function(){
		return this.$data;
	},
	
	saveData: function(data){
		if(data != this.$data){
			this.$data = data;
			this.$dirty = true;
			this.$notifyObservers();
		}
	},

	$loadData: function(){
		var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
		file.initWithPath(this.$persistenceFilename);
		if (file.exists()) {
			var is = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
			is.init(file, 0x01, 00004, null);
			var sis = Components.classes["@mozilla.org/scriptableinputstream;1"].createInstance(Components.interfaces.nsIScriptableInputStream);
			sis.init(is);
			this.$data = sis.read(sis.available());
			sis.close();
			is.close();
		}
	},

	$flushData: function(){
		if (this.$dirty && !this.$busy){
			this.busy = true;

			try{
				var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
				file.initWithPath(this.$persistenceFilename);
				if (!file.exists()) {
					file.create(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 420);
				}
				var outputStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
				outputStream.init(file, 0x04 | 0x08 | 0x20, 420, 0);
				outputStream.write(this.$data, this.$data.length);
				outputStream.close();
				this.$dirty = false;
			} catch(e){
			} finally{
				this.$busy = false;
			}

		}
	},

	/* notify all the observer that the persisted data has changed*/
	$notifyObservers: function(){
		for(var i in this.$obslist){
			this.$obslist[i].onPrefChange() ;
		}
	},

	registerStoreObserver: function(obs){
		for(var i in this.$obslist){
			if (this.$obslist[i] == obs) return;
		}
		
		this.$obslist.push(obs);
	},

	unregisterStoreObserver: function(obs){
		for(var i in this.$obslist){
			if (this.$obslist[i] == obs) this.$obslist.splice(i,1);
		}
	},
	

	// FF4 AddonManager Notifications Handlers
	onUninstalling: function(addon) {
		if (addon.id == this.$extid) {
			this.$uninstall = true;
		}
	},
	
	// FF4 AddonManager Notifications Handlers
	onOperationCancelled: function(addon) {
		if (addon.id == this.$extid) {
			this.$uninstall = (addon.pendingOperations & AddonManager.PENDING_UNINSTALL) != 0;
		}
	},
	
	cleanUserData: function(){
		var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
		file.initWithPath(this.$persistenceFilename);
		file.parent.remove(true);
	},

	observe: function(subject, topic, data) {

		/* part here removed //not FF 40+ compatible */
		
		if (topic == "quit-application-granted") {
			if (!this.$halted){
				this.$halted = true;
				this.$flushData();
			}

			if (this.$uninstall) {
				this.cleanUserData();
			}
		}
	}
};

function alert(msg){
	return Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
	.getService(Components.interfaces.nsIPromptService)
	.alert(null, "", msg);
}
